@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000 Paul Davis 
+    Copyright (C) 2000 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,24 +17,31 @@
 
 */
 
+#ifdef WAF_BUILD
+#include "gtk2ardour-config.h"
+#endif
+
 /* this file exists solely to break compilation dependencies that
    would connect changes to the mixer or editor objects.
 */
 
 #include <cstdio>
 
-#include <gtkmm/accelmap.h>
+#include "pbd/error.h"
 
-#include <pbd/error.h>
+#include "ardour/session.h"
 
+#include "actions.h"
 #include "ardour_ui.h"
 #include "public_editor.h"
+#include "meterbridge.h"
 #include "mixer_ui.h"
 #include "keyboard.h"
+#include "splash.h"
 #include "route_params_ui.h"
+#include "opts.h"
 #include "i18n.h"
 
-using namespace sigc;
 using namespace Gtk;
 using namespace PBD;
 
@@ -43,85 +50,54 @@ namespace ARDOUR {
 	class Route;
 }
 
-void
-ARDOUR_UI::shutdown ()
-{
-	if (session) {
-		/* we're exiting cleanly, so remove any auto-save data */
-		session->remove_pending_capture_state ();
-		session = 0;
-	}
-
-}
+using namespace ARDOUR;
 
 void
 ARDOUR_UI::we_have_dependents ()
-{
-	setup_keybindings ();
-	editor->UpdateAllTransportClocks.connect (mem_fun (*this, &ARDOUR_UI::update_transport_clocks));
-}
-
-static void 
-accel_map_changed (GtkAccelMap* map,
-		   gchar* path,
-		   guint  key,
-		   GdkModifierType mod,
-		   gpointer arg)
-{
-	static_cast<ARDOUR_UI*>(arg)->save_keybindings ();
-}
-
-void
-ARDOUR_UI::setup_keybindings ()
 {
 	install_actions ();
-	RedirectBox::register_actions ();
-	
-	cerr << "loading bindings from " << keybindings_path << endl;
+	ProcessorBox::register_actions ();
+	keyboard->setup_keybindings ();
+	editor->setup_tooltips ();
+	editor->UpdateAllTransportClocks.connect (sigc::mem_fun (*this, &ARDOUR_UI::update_transport_clocks));
 
-	try {
-		AccelMap::load (keybindings_path);
-	} catch (...) {
-		error << string_compose (_("Ardour key bindings file not found at \"%1\" or contains errors."), keybindings_path)
-		      << endmsg;
-	}
+	/* all actions are defined */
 
-	/* catch changes */
-
-	GtkAccelMap* accelmap = gtk_accel_map_get();
-	g_signal_connect (accelmap, "changed", (GCallback) accel_map_changed, this);
-
-	
+	ActionManager::enable_accelerators ();
+	ActionManager::load_menus (ARDOUR_COMMAND_LINE::menus_file);
 
+	editor->track_mixer_selection ();
+	mixer->track_editor_selection ();
 }
 
 void
 ARDOUR_UI::connect_dependents_to_session (ARDOUR::Session *s)
 {
-	editor->connect_to_session (s);
-	mixer->connect_to_session (s);
+	DisplaySuspender ds;
+	BootMessage (_("Setup Editor"));
+	editor->set_session (s);
+	BootMessage (_("Setup Mixer"));
+	mixer->set_session (s);
+	meterbridge->set_session (s);
 
 	/* its safe to do this now */
-	
+
+	BootMessage (_("Reload Session History"));
 	s->restore_history ("");
 }
 
-void
-ARDOUR_UI::goto_editor_window ()
-{
-	editor->show_window ();
-	editor->present();
-}
-void
-ARDOUR_UI::goto_mixer_window ()
-{
-	mixer->show_window ();
-	mixer->present();
-}
-
+/** The main editor window has been closed */
 gint
-ARDOUR_UI::exit_on_main_window_close (GdkEventAny *ev)
+ARDOUR_UI::exit_on_main_window_close (GdkEventAny * /*ev*/)
 {
+#ifdef TOP_MENUBAR
+	/* just hide the window, and return - the top menu stays up */
+	editor->hide ();
+	return TRUE;
+#else
+	/* time to get out of here */
 	finish();
 	return TRUE;
+#endif
 }
+
