@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000 Paul Davis 
+    Copyright (C) 2000 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,8 +23,12 @@
    is to cut down on the nasty compile times for these classes.
 */
 
+#include "actions.h"
 #include "ardour_ui.h"
 #include "mixer_ui.h"
+#include "meterbridge.h"
+
+#include "i18n.h"
 
 using namespace ARDOUR;
 using namespace PBD;
@@ -34,13 +38,33 @@ ARDOUR_UI::create_mixer ()
 
 {
 	try {
-		mixer = new Mixer_UI ();
-	} 
+		mixer = Mixer_UI::instance ();
+	}
 
 	catch (failed_constructor& err) {
 		return -1;
 	}
 
+	mixer->signal_window_state_event().connect (sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::main_window_state_event_handler), false));
+
+	return 0;
+}
+
+
+int
+ARDOUR_UI::create_meterbridge ()
+
+{
+	try {
+		meterbridge = Meterbridge::instance ();
+	}
+
+	catch (failed_constructor& err) {
+		return -1;
+	}
+
+	meterbridge->signal_window_state_event().connect (sigc::bind (sigc::mem_fun (*this, &ARDOUR_UI::main_window_state_event_handler), false));
+
 	return 0;
 }
 
