@@ -1,22 +1,25 @@
 #undef  Marker
 #define Marker FuckYouAppleAndYourLackOfNameSpaces
 
-#include <pbd/error.h>
-#include <ardour/audio_unit.h>
-#include <ardour/insert.h>
-
-#undef check // stupid gtk, stupid apple
-
 #include <gtkmm/button.h>
 #include <gdk/gdkquartz.h>
 
+#include "pbd/convert.h"
+#include "pbd/error.h"
+
+#include "ardour/audio_unit.h"
+#include "ardour/debug.h"
+#include "ardour/plugin_insert.h"
+
+#undef check // stupid gtk, stupid apple
+
 #include <gtkmm2ext/utils.h>
 
 #include "au_pluginui.h"
 #include "gui_thread.h"
 
-#include <appleutility/CAAudioUnit.h>
-#include <appleutility/CAComponent.h>
+#include "CAAudioUnit.h"
+#include "CAComponent.h"
 
 #import <AudioUnit/AUCocoaUIView.h>
 #import <CoreAudioKit/AUGenericView.h>
@@ -28,10 +31,14 @@
 #include "public_editor.h"
 #include "i18n.h"
 
+#ifdef COREAUDIO105
+#define ArdourCloseComponent CloseComponent
+#else
+#define ArdourCloseComponent AudioComponentInstanceDispose
+#endif
 using namespace ARDOUR;
 using namespace Gtk;
 using namespace Gtkmm2ext;
-using namespace sigc;
 using namespace std;
 using namespace PBD;
 
@@ -45,6 +52,29 @@ static const gchar* _automation_mode_strings[] = {
 	0
 };
 
+static void
+dump_view_tree (NSView* view, int depth, int maxdepth)
+{
+	NSArray* subviews = [view subviews];
+	unsigned long cnt = [subviews count];
+
+	for (int d = 0; d < depth; d++) {
+		cerr << '\t';
+	}
+	NSRect frame = [view frame];
+	cerr << " view @ " <<  frame.origin.x << ", " << frame.origin.y
+		<< ' ' << frame.size.width << " x " << frame.size.height
+		<< endl;
+
+	if (depth >= maxdepth) {
+		return;
+	}
+	for (unsigned long i = 0; i < cnt; ++i) {
+		NSView* subview = [subviews objectAtIndex:i];
+		dump_view_tree (subview, depth+1, maxdepth);
+	}
+}
+
 @implementation NotificationObject
 
 - (NotificationObject*) initWithPluginUI: (AUPluginUI*) apluginui andCocoaParent: (NSWindow*) cp andTopLevelParent: (NSWindow*) tlp
@@ -53,23 +83,28 @@ static const gchar* _automation_mode_strings[] = {
 
 	if (self) {
 		plugin_ui = apluginui;
-		cocoa_parent = cp;
 		top_level_parent = tlp;
 
-		[[NSNotificationCenter defaultCenter] addObserver:self
-		 selector:@selector(cocoaParentActivationHandler:)
-		 name:NSWindowDidBecomeMainNotification
-		 object:nil];
+		if (cp) {
+			cocoa_parent = cp;
 
-		[[NSNotificationCenter defaultCenter] addObserver:self
-		 selector:@selector(cocoaParentBecameKeyHandler:)
-		 name:NSWindowDidBecomeKeyNotification
-		 object:nil];
+			[[NSNotificationCenter defaultCenter]
+			     addObserver:self
+			        selector:@selector(cocoaParentActivationHandler:)
+			            name:NSWindowDidBecomeMainNotification
+			          object:NULL];
+
+			[[NSNotificationCenter defaultCenter]
+			     addObserver:self
+			        selector:@selector(cocoaParentBecameKeyHandler:)
+			            name:NSWindowDidBecomeKeyNotification
+			          object:NULL];
+		}
 	}
 
 	return self;
 }
-		
+
 - (void)cocoaParentActivationHandler:(NSNotification *)notification
 {
 	NSWindow* notification_window = (NSWindow *)[notification object];
@@ -80,32 +115,27 @@ static const gchar* _automation_mode_strings[] = {
 		} else {
 			plugin_ui->deactivate();
 		}
-	} 
+	}
 }
 
 - (void)cocoaParentBecameKeyHandler:(NSNotification *)notification
 {
 	NSWindow* notification_window = (NSWindow *)[notification object];
 
-	cerr << "KeyNotification Handler\n";
-
 	if (top_level_parent == notification_window || cocoa_parent == notification_window) {
-		cerr << "\tKeyHandler, top level parent is key: " <<  ([top_level_parent isKeyWindow] ? "yes" : "no") << endl;
-		cerr << "\tKeyHandler, cocoa parent is key: " <<  ([cocoa_parent isKeyWindow] ? "yes" : "no") << endl;
-		
 		if ([notification_window isKeyWindow]) {
-			cerr << "\t\tActivating plugin UI\n";
 			plugin_ui->activate();
 		} else {
-			cerr << "\t\tDeActivating plugin UI\n";
 			plugin_ui->deactivate();
 		}
-	} else {
-		cerr << "\tsome other window become Key (" << notification_window << ") CP is " << cocoa_parent << "\n";
 	}
 }
 
-
+- (void)auViewResized:(NSNotification *)notification
+{
+	(void) notification; // stop complaints about unusued argument
+	plugin_ui->cocoa_view_resized();
+}
 
 @end
 
@@ -113,12 +143,20 @@ AUPluginUI::AUPluginUI (boost::shared_ptr<PluginInsert> insert)
 	: PlugUIBase (insert)
 	, automation_mode_label (_("Automation"))
 	, preset_label (_("Presets"))
-	
+	, mapped (false)
+	, resizable (false)
+	, min_width (0)
+	, min_height (0)
+	, req_width (0)
+	, req_height (0)
+	, alo_width (0)
+	, alo_height (0)
+
 {
 	if (automation_mode_strings.empty()) {
 		automation_mode_strings = I18N (_automation_mode_strings);
 	}
-	
+
 	set_popdown_strings (automation_mode_selector, automation_mode_strings);
 	automation_mode_selector.set_active_text (automation_mode_strings.front());
 
@@ -133,10 +171,21 @@ AUPluginUI::AUPluginUI (boost::shared_ptr<PluginInsert> insert)
 
 	smaller_hbox->set_spacing (6);
 	smaller_hbox->pack_start (preset_label, false, false, 4);
-	smaller_hbox->pack_start (preset_combo, false, false);
+	smaller_hbox->pack_start (_preset_modified, false, false);
+	smaller_hbox->pack_start (_preset_combo, false, false);
+	smaller_hbox->pack_start (add_button, false, false);
+#if 0
+	/* Ardour does not currently allow to overwrite existing presets
+	 * see save_property_list() in audio_unit.cc
+	 */
 	smaller_hbox->pack_start (save_button, false, false);
+#endif
+#if 0
+	/* one day these might be useful with an AU plugin, but not yet */
 	smaller_hbox->pack_start (automation_mode_label, false, false);
 	smaller_hbox->pack_start (automation_mode_selector, false, false);
+#endif
+	smaller_hbox->pack_start (reset_button, false, false);
 	smaller_hbox->pack_start (bypass_button, false, true);
 
 	VBox* v1_box = manage (new VBox);
@@ -154,73 +203,95 @@ AUPluginUI::AUPluginUI (boost::shared_ptr<PluginInsert> insert)
 
 	set_spacing (6);
 	pack_start (top_box, false, false);
-	pack_start (low_box, false, false);
+	pack_start (low_box, true, true);
 
 	preset_label.show ();
-	preset_combo.show ();
+	_preset_combo.show ();
 	automation_mode_label.show ();
 	automation_mode_selector.show ();
 	bypass_button.show ();
 	top_box.show ();
 	low_box.show ();
 
-	_activating_from_app = false;
 	cocoa_parent = 0;
-	_notify = 0;
 	cocoa_window = 0;
+
+#ifdef WITH_CARBON
+	_activating_from_app = false;
+	_notify = 0;
 	au_view = 0;
-	packView = 0;
 	editView = 0;
+	carbon_window = 0;
+#endif
 
 	/* prefer cocoa, fall back to cocoa, but use carbon if its there */
 
 	if (test_cocoa_view_support()) {
 		create_cocoa_view ();
+#ifdef WITH_CARBON
 	} else if (test_carbon_view_support()) {
 		create_carbon_view ();
+#endif
 	} else {
 		create_cocoa_view ();
 	}
 
+	low_box.add_events (Gdk::VISIBILITY_NOTIFY_MASK | Gdk::EXPOSURE_MASK);
+
 	low_box.signal_realize().connect (mem_fun (this, &AUPluginUI::lower_box_realized));
+	low_box.signal_visibility_notify_event ().connect (mem_fun (this, &AUPluginUI::lower_box_visibility_notify));
+	if (au_view) {
+		low_box.signal_size_request ().connect (mem_fun (this, &AUPluginUI::lower_box_size_request));
+		low_box.signal_size_allocate ().connect (mem_fun (this, &AUPluginUI::lower_box_size_allocate));
+		low_box.signal_map ().connect (mem_fun (this, &AUPluginUI::lower_box_map));
+		low_box.signal_unmap ().connect (mem_fun (this, &AUPluginUI::lower_box_unmap));
+		low_box.signal_expose_event ().connect (mem_fun (this, &AUPluginUI::lower_box_expose));
+	}
 }
 
 AUPluginUI::~AUPluginUI ()
 {
+	if (_notify) {
+		[[NSNotificationCenter defaultCenter] removeObserver:_notify];
+	}
+
 	if (cocoa_parent) {
 		NSWindow* win = get_nswindow();
-		[[NSNotificationCenter defaultCenter] removeObserver:_notify];
 		[win removeChildWindow:cocoa_parent];
+	}
 
-	} else if (carbon_window) {
+#ifdef WITH_CARBON
+	if (carbon_window) {
 		/* not parented, just overlaid on top of our window */
 		DisposeWindow (carbon_window);
 	}
+#endif
 
 	if (editView) {
-		CloseComponent (editView);
+		ArdourCloseComponent (editView);
 	}
 
-	if (packView) {
+	if (au_view) {
 		/* remove whatever we packed into low_box so that GTK doesn't
 		   mess with it.
-		*/
+		 */
 
-		[packView removeFromSuperview];
+		[au_view removeFromSuperview];
 	}
 }
 
 bool
 AUPluginUI::test_carbon_view_support ()
 {
+#ifdef WITH_CARBON
 	bool ret = false;
-	
+
 	carbon_descriptor.componentType = kAudioUnitCarbonViewComponentType;
 	carbon_descriptor.componentSubType = 'gnrc';
 	carbon_descriptor.componentManufacturer = 'appl';
 	carbon_descriptor.componentFlags = 0;
 	carbon_descriptor.componentFlagsMask = 0;
-	
+
 	OSStatus err;
 
 	// ask the AU for its first editor component
@@ -239,8 +310,11 @@ AUPluginUI::test_carbon_view_support ()
 	}
 
 	return ret;
+#else
+	return false;
+#endif
 }
-	
+
 bool
 AUPluginUI::test_cocoa_view_support ()
 {
@@ -249,7 +323,7 @@ AUPluginUI::test_cocoa_view_support ()
 	OSStatus err = AudioUnitGetPropertyInfo(*au->get_au(),
 						kAudioUnitProperty_CocoaUI, kAudioUnitScope_Global,
 						0, &dataSize, &isWritable);
-	
+
 	return dataSize > 0 && err == noErr;
 }
 
@@ -268,27 +342,32 @@ AUPluginUI::plugin_class_valid (Class pluginClass)
 int
 AUPluginUI::create_cocoa_view ()
 {
-	BOOL wasAbleToLoadCustomView = NO;
+	bool wasAbleToLoadCustomView = false;
 	AudioUnitCocoaViewInfo* cocoaViewInfo = NULL;
 	UInt32               numberOfClasses = 0;
 	UInt32     dataSize;
 	Boolean    isWritable;
 	NSString*	    factoryClassName = 0;
-	NSURL*	            CocoaViewBundlePath;
+	NSURL*	            CocoaViewBundlePath = NULL;
 
 	OSStatus result = AudioUnitGetPropertyInfo (*au->get_au(),
 						    kAudioUnitProperty_CocoaUI,
-						    kAudioUnitScope_Global, 
+						    kAudioUnitScope_Global,
 						    0,
 						    &dataSize,
 						    &isWritable );
 
 	numberOfClasses = (dataSize - sizeof(CFURLRef)) / sizeof(CFStringRef);
-	
+
 	// Does view have custom Cocoa UI?
-	
+
 	if ((result == noErr) && (numberOfClasses > 0) ) {
+
+		DEBUG_TRACE(DEBUG::AudioUnits,
+			    string_compose ( "based on %1, there are %2 cocoa UI classes\n", dataSize, numberOfClasses));
+
 		cocoaViewInfo = (AudioUnitCocoaViewInfo *)malloc(dataSize);
+
 		if(AudioUnitGetProperty(*au->get_au(),
 					kAudioUnitProperty_CocoaUI,
 					kAudioUnitScope_Global,
@@ -297,12 +376,17 @@ AUPluginUI::create_cocoa_view ()
 					&dataSize) == noErr) {
 
 			CocoaViewBundlePath	= (NSURL *)cocoaViewInfo->mCocoaAUViewBundleLocation;
-			
+
 			// we only take the first view in this example.
 			factoryClassName	= (NSString *)cocoaViewInfo->mCocoaAUViewClass[0];
 
+			DEBUG_TRACE (DEBUG::AudioUnits, string_compose ("the factory name is %1 bundle is %2\n",
+									[factoryClassName UTF8String], CocoaViewBundlePath));
+
 		} else {
 
+			DEBUG_TRACE (DEBUG::AudioUnits, string_compose ("No cocoaUI property cocoaViewInfo = %1\n", cocoaViewInfo));
+
 			if (cocoaViewInfo != NULL) {
 				free (cocoaViewInfo);
 				cocoaViewInfo = NULL;
@@ -310,98 +394,129 @@ AUPluginUI::create_cocoa_view ()
 		}
 	}
 
-	NSRect crect = { { 0, 0 }, { 1, 1} };
-
 	// [A] Show custom UI if view has it
 
 	if (CocoaViewBundlePath && factoryClassName) {
 		NSBundle *viewBundle  	= [NSBundle bundleWithPath:[CocoaViewBundlePath path]];
-		if (viewBundle == nil) {
+
+		DEBUG_TRACE (DEBUG::AudioUnits, string_compose ("tried to create bundle, result = %1\n", viewBundle));
+
+		if (viewBundle == NULL) {
 			error << _("AUPluginUI: error loading AU view's bundle") << endmsg;
 			return -1;
 		} else {
 			Class factoryClass = [viewBundle classNamed:factoryClassName];
+			DEBUG_TRACE (DEBUG::AudioUnits, string_compose ("tried to create factory class, result = %1\n", factoryClass));
 			if (!factoryClass) {
 				error << _("AUPluginUI: error getting AU view's factory class from bundle") << endmsg;
 				return -1;
 			}
-			
+
 			// make sure 'factoryClass' implements the AUCocoaUIBase protocol
 			if (!plugin_class_valid (factoryClass)) {
 				error << _("AUPluginUI: U view's factory class does not properly implement the AUCocoaUIBase protocol") << endmsg;
 				return -1;
 			}
 			// make a factory
-			id factoryInstance = [[[factoryClass alloc] init] autorelease];
-			if (factoryInstance == nil) {
+			id factory = [[[factoryClass alloc] init] autorelease];
+			if (factory == NULL) {
 				error << _("AUPluginUI: Could not create an instance of the AU view factory") << endmsg;
 				return -1;
 			}
 
+			DEBUG_TRACE (DEBUG::AudioUnits, "got a factory instance\n");
+
 			// make a view
-			au_view = [factoryInstance uiViewForAudioUnit:*au->get_au() withSize:crect.size];
-			
+			au_view = [factory uiViewForAudioUnit:*au->get_au() withSize:NSZeroSize];
+
+			DEBUG_TRACE (DEBUG::AudioUnits, string_compose ("view created @ %1\n", au_view));
+
 			// cleanup
 			[CocoaViewBundlePath release];
 			if (cocoaViewInfo) {
 				UInt32 i;
 				for (i = 0; i < numberOfClasses; i++)
 					CFRelease(cocoaViewInfo->mCocoaAUViewClass[i]);
-				
+
 				free (cocoaViewInfo);
 			}
-			wasAbleToLoadCustomView = YES;
+			wasAbleToLoadCustomView = true;
 		}
 	}
 
 	if (!wasAbleToLoadCustomView) {
 		// load generic Cocoa view
+		DEBUG_TRACE (DEBUG::AudioUnits, string_compose ("Loading generic view using %1 -> %2\n", au,
+								au->get_au()));
 		au_view = [[AUGenericView alloc] initWithAudioUnit:*au->get_au()];
-		[(AUGenericView *)au_view setShowsExpertParameters:YES];
+		DEBUG_TRACE (DEBUG::AudioUnits, string_compose ("view created @ %1\n", au_view));
+		[(AUGenericView *)au_view setShowsExpertParameters:1];
 	}
 
-	packView = au_view;
+	// Get the initial size of the new AU View's frame
+	NSRect  frame = [au_view frame];
+	min_width  = req_width  = CGRectGetWidth(NSRectToCGRect(frame));
+	min_height = req_height = CGRectGetHeight(NSRectToCGRect(frame));
+	resizable  = [au_view autoresizingMask];
 
-	// Get the size of the new AU View's frame 
-	
-	NSRect packFrame;
-	packFrame = [au_view frame];
-	prefwidth = packFrame.size.width;
-	prefheight = packFrame.size.height;
+	low_box.queue_resize ();
 
 	return 0;
 }
 
+void
+AUPluginUI::cocoa_view_resized ()
+{
+	if (!mapped || alo_width == 0 || alo_height == 0 || !resizable) {
+		return;
+	}
+	/* check for self-resizing plugins (e.g expand settings in AUSampler)
+	 * if the widget expands it moves its y-offset (cocoa y-axis points towards the top)
+	 */
+	NSRect new_au_frame = [au_view frame];
+
+	//float dx = last_au_frame.origin.x - new_au_frame.origin.x;
+	float dy = last_au_frame.origin.y - new_au_frame.origin.y;
+	//req_width += dx;
+	req_height += dy;
+	if (req_width < min_width) req_width = min_width;
+	if (req_height < min_height) req_height = min_height;
+
+	last_au_frame = new_au_frame;
+	low_box.queue_resize ();
+}
+
 int
 AUPluginUI::create_carbon_view ()
 {
+#ifdef WITH_CARBON
 	OSStatus err;
 	ControlRef root_control;
 
 	Component editComponent = FindNextComponent(NULL, &carbon_descriptor);
-	
+
 	OpenAComponent(editComponent, &editView);
 	if (!editView) {
 		error << _("AU Carbon view: cannot open AU Component") << endmsg;
 		return -1;
 	}
-	
+
 	Rect r = { 100, 100, 100, 100 };
 	WindowAttributes attr = WindowAttributes (kWindowStandardHandlerAttribute |
 						  kWindowCompositingAttribute|
 						  kWindowNoShadowAttribute|
 						  kWindowNoTitleBarAttribute);
 
-	if ((err = CreateNewWindow(kDocumentWindowClass, attr, &r, &carbon_window)) != noErr) {
+	if ((err = CreateNewWindow(kUtilityWindowClass, attr, &r, &carbon_window)) != noErr) {
 		error << string_compose (_("AUPluginUI: cannot create carbon window (err: %1)"), err) << endmsg;
-	        CloseComponent (editView);
+	        ArdourCloseComponent (editView);
 		return -1;
 	}
-	
+
 	if ((err = GetRootControl(carbon_window, &root_control)) != noErr) {
 		error << string_compose (_("AUPlugin: cannot get root control of carbon window (err: %1)"), err) << endmsg;
 		DisposeWindow (carbon_window);
-	        CloseComponent (editView);
+	        ArdourCloseComponent (editView);
 		return -1;
 	}
 
@@ -412,7 +527,7 @@ AUPluginUI::create_carbon_view ()
 	if ((err = AudioUnitCarbonViewCreate (editView, *au->get_au(), carbon_window, root_control, &location, &size, &viewPane)) != noErr) {
 		error << string_compose (_("AUPluginUI: cannot create carbon plugin view (err: %1)"), err) << endmsg;
 		DisposeWindow (carbon_window);
-	        CloseComponent (editView);
+	        ArdourCloseComponent (editView);
 		return -1;
 	}
 
@@ -423,13 +538,17 @@ AUPluginUI::create_carbon_view ()
 	size.x = bounds.right-bounds.left;
 	size.y = bounds.bottom-bounds.top;
 
-	prefwidth = (int) (size.x + 0.5);
-	prefheight = (int) (size.y + 0.5);
+	req_width = (int) (size.x + 0.5);
+	req_height = (int) (size.y + 0.5);
 
-	SizeWindow (carbon_window, prefwidth, prefheight,  true);
-	low_box.set_size_request (prefwidth, prefheight);
+	SizeWindow (carbon_window, req_width, req_height,  true);
+	low_box.set_size_request (req_width, req_height);
 
 	return 0;
+#else
+	error << _("AU Carbon GUI is not supported.") << endmsg;
+	return -1;
+#endif
 }
 
 NSWindow*
@@ -455,34 +574,35 @@ AUPluginUI::get_nswindow ()
 void
 AUPluginUI::activate ()
 {
+#ifdef WITH_CARBON
 	ActivateWindow (carbon_window, TRUE);
-	// [cocoa_parent makeKeyAndOrderFront:nil];
+#endif
 }
 
 void
 AUPluginUI::deactivate ()
 {
+#ifdef WITH_CARBON
 	ActivateWindow (carbon_window, FALSE);
+#endif
 }
 
 int
 AUPluginUI::parent_carbon_window ()
 {
+#ifdef WITH_CARBON
 	NSWindow* win = get_nswindow ();
-	int x, y;
+	Rect windowStructureBoundsRect;
 
 	if (!win) {
 		return -1;
 	}
 
-	Gtk::Container* toplevel = get_toplevel();
+	/* figure out where the cocoa parent window is in carbon-coordinate space, which
+	   differs from both cocoa-coordinate space and GTK-coordinate space
+	*/
 
-	if (!toplevel || !toplevel->is_toplevel()) {
-		error << _("AUPluginUI: no top level window!") << endmsg;
-		return -1;
-	}
-	
-	toplevel->get_window()->get_root_origin (x, y);
+	GetWindowBounds((WindowRef) [win windowRef], kWindowStructureRgn, &windowStructureBoundsRect);
 
 	/* compute how tall the title bar is, because we have to offset the position of the carbon window
 	   by that much.
@@ -495,7 +615,11 @@ AUPluginUI::parent_carbon_window ()
 
 	int packing_extra = 6; // this is the total vertical packing in our top level window
 
-	MoveWindow (carbon_window, x, y + titlebar_height + top_box.get_height() + packing_extra, false);
+	/* move into position, based on parent window position */
+	MoveWindow (carbon_window,
+		    windowStructureBoundsRect.left,
+		    windowStructureBoundsRect.top + titlebar_height + top_box.get_height() + packing_extra,
+		    false);
 	ShowWindow (carbon_window);
 
 	// create the cocoa window for the carbon one and make it visible
@@ -503,45 +627,78 @@ AUPluginUI::parent_carbon_window ()
 
 	SetWindowActivationScope (carbon_window, kWindowActivationScopeNone);
 
-	_notify = [ [NotificationObject alloc] initWithPluginUI:this andCocoaParent:cocoa_parent andTopLevelParent:win ]; 
+	_notify = [ [NotificationObject alloc] initWithPluginUI:this andCocoaParent:cocoa_parent andTopLevelParent:win ];
 
 	[win addChildWindow:cocoa_parent ordered:NSWindowAbove];
+	[win setAutodisplay:1]; // turn of GTK stuff for this window
 
 	return 0;
-}	
+#else
+	return -1;
+#endif
+}
 
 int
 AUPluginUI::parent_cocoa_window ()
 {
 	NSWindow* win = get_nswindow ();
-	NSRect packFrame;
 
 	if (!win) {
 		return -1;
 	}
 
-	[win setAutodisplay:YES]; // turn of GTK stuff for this window
-
-	Gtk::Container* toplevel = get_toplevel();
-
-	if (!toplevel || !toplevel->is_toplevel()) {
-		error << _("AUPluginUI: no top level window!") << endmsg;
-		return -1;
-	}
-	
-	// Get the size of the new AU View's frame 
-	packFrame = [au_view frame];
+	//[win setAutodisplay:1]; // turn off GTK stuff for this window
 
 	NSView* view = gdk_quartz_window_get_nsview (low_box.get_window()->gobj());
+	[view addSubview:au_view];
 
-	[view setFrame:packFrame];
-	[view addSubview:packView]; 
+	gint xx, yy;
+	gtk_widget_translate_coordinates(
+			GTK_WIDGET(low_box.gobj()),
+			GTK_WIDGET(low_box.get_parent()->gobj()),
+			8, 6, &xx, &yy);
+	[au_view setFrame:NSMakeRect(xx, yy, req_width, req_height)];
 
-	low_box.set_size_request (packFrame.size.width, packFrame.size.height);
+	last_au_frame = [au_view frame];
+	// watch for size changes of the view
+	_notify = [ [NotificationObject alloc] initWithPluginUI:this andCocoaParent:NULL andTopLevelParent:win ];
+
+	[[NSNotificationCenter defaultCenter] addObserver:_notify
+		selector:@selector(auViewResized:) name:NSViewFrameDidChangeNotification
+		object:au_view];
 
 	return 0;
 }
 
+void
+AUPluginUI::grab_focus()
+{
+	if (au_view) {
+		[au_view becomeFirstResponder];
+	}
+}
+void
+AUPluginUI::forward_key_event (GdkEventKey* ev)
+{
+	NSEvent* nsevent = gdk_quartz_event_get_nsevent ((GdkEvent*)ev);
+
+	if (au_view && nsevent) {
+
+		/* filter on nsevent type here because GDK massages FlagsChanged
+		   messages into GDK_KEY_{PRESS,RELEASE} but Cocoa won't
+		   handle a FlagsChanged message as a keyDown or keyUp
+		*/
+
+		if ([nsevent type] == NSKeyDown) {
+			[[[au_view window] firstResponder] keyDown:nsevent];
+		} else if ([nsevent type] == NSKeyUp) {
+			[[[au_view window] firstResponder] keyUp:nsevent];
+		} else if ([nsevent type] == NSFlagsChanged) {
+			[[[au_view window] firstResponder] flagsChanged:nsevent];
+		}
+	}
+}
+
 void
 AUPluginUI::on_realize ()
 {
@@ -551,7 +708,7 @@ AUPluginUI::on_realize ()
 
 	NSWindow* win = get_nswindow ();
 	if (win) {
-		[win setShowsResizeIndicator:NO];
+		[win setShowsResizeIndicator:0];
 	}
 }
 
@@ -566,24 +723,126 @@ AUPluginUI::lower_box_realized ()
 }
 
 bool
-AUPluginUI::on_map_event (GdkEventAny* ev)
+AUPluginUI::lower_box_visibility_notify (GdkEventVisibility* ev)
 {
+#ifdef WITH_CARBON
+	if (carbon_window  && ev->state != GDK_VISIBILITY_UNOBSCURED) {
+		ShowWindow (carbon_window);
+		ActivateWindow (carbon_window, TRUE);
+		return true;
+	}
+#endif
 	return false;
 }
 
+void
+AUPluginUI::update_view_size ()
+{
+	if (!mapped || alo_width == 0 || alo_height == 0) {
+		return;
+	}
+	gint xx, yy;
+	gtk_widget_translate_coordinates(
+			GTK_WIDGET(low_box.gobj()),
+			GTK_WIDGET(low_box.get_parent()->gobj()),
+			8, 6, &xx, &yy);
+
+	[[NSNotificationCenter defaultCenter] removeObserver:_notify
+		name:NSViewFrameDidChangeNotification
+		object:au_view];
+
+	if (!resizable) {
+		xx += (alo_width - req_width) * .5;
+		[au_view setFrame:NSMakeRect(xx, yy, req_width, req_height)];
+	} else {
+		/* this mitigates issues with plugins that resize themselves
+		 * depending on visible options (e.g AUSampler)
+		 * since the OSX y-axis points upwards, the plugin adjusts its
+		 * own y-offset if the view expands to the bottom to accomodate
+		 * subviews inside the main view.
+		 */
+		[au_view setAutoresizesSubviews:0];
+		[au_view setFrame:NSMakeRect(xx, yy, alo_width, alo_height)];
+		[au_view setAutoresizesSubviews:1];
+		[au_view setNeedsDisplay:1];
+	}
+
+	last_au_frame = [au_view frame];
+
+	[[NSNotificationCenter defaultCenter]
+	     addObserver:_notify
+	        selector:@selector(auViewResized:) name:NSViewFrameDidChangeNotification
+	          object:au_view];
+}
+
+void
+AUPluginUI::lower_box_map ()
+{
+	mapped = true;
+	[au_view setHidden:0];
+	update_view_size ();
+}
+
+void
+AUPluginUI::lower_box_unmap ()
+{
+	mapped = false;
+	[au_view setHidden:1];
+}
+
+void
+AUPluginUI::lower_box_size_request (GtkRequisition* requisition)
+{
+	requisition->width  = req_width;
+	requisition->height = req_height;
+}
+
+void
+AUPluginUI::lower_box_size_allocate (Gtk::Allocation& allocation)
+{
+	alo_width  = allocation.get_width ();
+	alo_height = allocation.get_height ();
+	update_view_size ();
+}
+
+gboolean
+AUPluginUI::lower_box_expose (GdkEventExpose* event)
+{
+#if 0 // AU view magically redraws by itself
+	[au_view drawRect:NSMakeRect(event->area.x,
+			event->area.y,
+			event->area.width,
+			event->area.height)];
+#endif
+	/* hack to keep ardour responsive
+	 * some UIs (e.g Addictive Drums) completely hog the CPU
+	 */
+	ARDOUR::GUIIdle();
+
+	return true;
+}
+
 void
 AUPluginUI::on_window_hide ()
 {
+#ifdef WITH_CARBON
 	if (carbon_window) {
 		HideWindow (carbon_window);
 		ActivateWindow (carbon_window, FALSE);
 	}
-
+#endif
 	hide_all ();
+
+#if 0
+	NSArray* wins = [NSApp windows];
+	for (uint32_t i = 0; i < [wins count]; i++) {
+		id win = [wins objectAtIndex:i];
+	}
+#endif
 }
 
 bool
-AUPluginUI::on_window_show (const Glib::ustring& title)
+AUPluginUI::on_window_show (const string& /*title*/)
 {
 	/* this is idempotent so just call it every time we show the window */
 
@@ -591,22 +850,24 @@ AUPluginUI::on_window_show (const Glib::ustring& title)
 
 	show_all ();
 
+#ifdef WITH_CARBON
 	if (carbon_window) {
 		ShowWindow (carbon_window);
 		ActivateWindow (carbon_window, TRUE);
 	}
+#endif
 
 	return true;
 }
 
 bool
-AUPluginUI::start_updating (GdkEventAny* any)
+AUPluginUI::start_updating (GdkEventAny*)
 {
 	return false;
 }
 
 bool
-AUPluginUI::stop_updating (GdkEventAny* any)
+AUPluginUI::stop_updating (GdkEventAny*)
 {
 	return false;
 }
@@ -619,19 +880,4 @@ create_au_gui (boost::shared_ptr<PluginInsert> plugin_insert, VBox** box)
 	return aup;
 }
 
-bool
-AUPluginUI::on_focus_in_event (GdkEventFocus* ev)
-{
-	//cerr << "au plugin focus in\n";
-	//Keyboard::magic_widget_grab_focus ();
-	return false;
-}
-
-bool
-AUPluginUI::on_focus_out_event (GdkEventFocus* ev)
-{
-	//cerr << "au plugin focus out\n";
-	//Keyboard::magic_widget_drop_focus ();
-	return false;
-}
 
