@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000-2006 Paul Davis 
+    Copyright (C) 2000-2006 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,21 +31,19 @@
 #include <gtkmm2ext/selector.h>
 #include <list>
 
-#include <ardour/types.h>
+#include "ardour/types.h"
 
 #include "ardour_dialog.h"
 #include "route_ui.h"
 #include "enums.h"
 #include "editing.h"
 #include "route_time_axis.h"
-#include "canvas.h"
 
 namespace ARDOUR {
 	class Session;
-	class AudioDiskstream;
 	class RouteGroup;
-	class Redirect;
-	class Insert;
+	class IOProcessor;
+	class Processor;
 	class Location;
 	class AudioPlaylist;
 }
@@ -66,73 +64,38 @@ class AutomationTimeAxisView;
 class AudioTimeAxisView : public RouteTimeAxisView
 {
   public:
- 	AudioTimeAxisView (PublicEditor&, ARDOUR::Session&, boost::shared_ptr<ARDOUR::Route>, ArdourCanvas::Canvas& canvas);
+ 	AudioTimeAxisView (PublicEditor&, ARDOUR::Session*, ArdourCanvas::Canvas& canvas);
  	virtual ~AudioTimeAxisView ();
-	
+
+	void set_route (boost::shared_ptr<ARDOUR::Route>);
+
 	AudioStreamView* audio_view();
 
-	void set_show_waveforms (bool yn);
 	void set_show_waveforms_recording (bool yn);
-	void show_all_xfades ();
-	void hide_all_xfades ();
-	void hide_dependent_views (TimeAxisViewItem&);
-	void reveal_dependent_views (TimeAxisViewItem&);
-		
+
 	/* Overridden from parent to store display state */
 	guint32 show_at (double y, int& nth, Gtk::VBox *parent);
-	void hide ();
-	
-	int set_state (const XMLNode&);
-	XMLNode* get_child_xml_node (const string & childname);
+
+	void create_automation_child (const Evoral::Parameter& param, bool show);
 
 	void first_idle ();
 
-	void set_waveform_shape (Editing::WaveformShape);
-	void set_waveform_scale (Editing::WaveformScale);
-
   private:
 	friend class AudioStreamView;
 	friend class AudioRegionView;
-	
+
 	void route_active_changed ();
 
-	void build_automation_action_menu ();
-	void append_extra_display_menu_items ();
-	
-	void toggle_show_waveforms ();
-	void toggle_waveforms ();
+	Gtk::Menu* build_mode_menu();
+	void build_automation_action_menu (bool);
 
-	void show_all_automation ();
-	void show_existing_automation ();
-	void hide_all_automation ();
+	void show_all_automation (bool apply_to_selection = false);
+	void show_existing_automation (bool apply_to_selection = false);
+	void hide_all_automation (bool apply_to_selection = false);
 
-	void add_gain_automation_child ();
-	void add_pan_automation_child ();
-	void add_parameter_automation_child ();
+	void hide ();
 
-	void toggle_gain_track ();
-	void toggle_pan_track ();
-
-	void gain_hidden ();
-	void pan_hidden ();
-
-	void update_pans ();
 	void update_control_names ();
-
-	AutomationTimeAxisView* gain_track;
-	AutomationTimeAxisView* pan_track;
-
-	// Set from XML so context menu automation buttons can be correctly initialized
-	bool show_gain_automation;
-	bool show_pan_automation;
-	
-	Gtk::CheckMenuItem* waveform_item;
-	Gtk::RadioMenuItem* traditional_item;
-	Gtk::RadioMenuItem* rectified_item;
-	Gtk::RadioMenuItem* linearscale_item;
-	Gtk::RadioMenuItem* logscale_item;
-	Gtk::CheckMenuItem* gain_automation_item;
-	Gtk::CheckMenuItem* pan_automation_item;
 };
 
 #endif /* __ardour_audio_time_axis_h__ */
