@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2003 Paul Davis 
+    Copyright (C) 2003 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,11 +21,17 @@
 #define __ardour_gtk_axis_view_h__
 
 #include <list>
+#include <boost/unordered_map.hpp>
 
 #include <gtkmm/label.h>
 #include <gdkmm/color.h>
 
-#include <pbd/xml++.h>
+#include "pbd/xml++.h"
+#include "pbd/signals.h"
+
+#include "ardour/session_handle.h"
+
+#include "gui_object.h"
 #include "prompter.h"
 #include "selectable.h"
 
@@ -37,39 +43,50 @@ namespace ARDOUR {
  * AxisView defines the abstract base class for time-axis trackviews and routes.
  *
  */
-class AxisView : public virtual Selectable
+class AxisView : public virtual Selectable, public PBD::ScopedConnectionList, public ARDOUR::SessionHandlePtr
 {
   public:
-	/**
-	 * Returns the current 'Track' Color
-	 *
-	 * @return the current Track Color
-	 */
-	Gdk::Color color() const { return _color; }
+	/** @return the track's own color */
+	Gdk::Color color () const { return _color; }
 
-	ARDOUR::Session& session() const { return _session; }
+	ARDOUR::Session* session() const { return _session; }
 
-	virtual string name() const = 0;
+	virtual std::string name() const = 0;
 
-	virtual bool marked_for_display() const { return _marked_for_display; }
-
-	virtual void set_marked_for_display (bool yn) {
-		if (yn != _marked_for_display) {
-			_marked_for_display = yn;
-		}
-	}
-	
 	sigc::signal<void> Hiding;
-	sigc::signal<void> GoingAway;
 
 	void set_old_order_key (uint32_t ok) { _old_order_key = ok; }
 	uint32_t old_order_key() const { return _old_order_key; }
 
+	virtual std::string state_id() const = 0;
+	/* for now, we always return properties in string form.
+	 */
+	std::string gui_property (const std::string& property_name) const;
+
+	template<typename T> void set_gui_property (const std::string& property_name, const T& value) {
+		std::stringstream s;
+		s << value;
+		property_hashtable.erase(property_name);
+		property_hashtable.emplace(property_name, s.str());
+		gui_object_state().set_property<T> (state_id(), property_name, value);
+	}
+
+	void cleanup_gui_properties () {
+		/* remove related property node from the GUI state */
+		gui_object_state().remove_node (state_id());
+		property_hashtable.clear ();
+	}
+
+	bool marked_for_display () const;
+	virtual bool set_marked_for_display (bool);
+
+	static GUIObjectState& gui_object_state();
+	void clear_property_cache() { property_hashtable.clear(); }
+
   protected:
 
-	AxisView (ARDOUR::Session& sess);
+	AxisView (ARDOUR::Session* sess);
 	virtual ~AxisView();
-	
 
 	/**
 	 * Generate a new random TrackView color, unique from those colors already used.
@@ -78,17 +95,14 @@ class AxisView : public virtual Selectable
 	 */
 	static Gdk::Color unique_random_color();
 
-
-	ARDOUR::Session& _session;
 	Gdk::Color _color;
 
-	static list<Gdk::Color> used_colors;
+	static std::list<Gdk::Color> used_colors;
 
 	Gtk::Label name_label;
 
-	bool _marked_for_display;
+	mutable boost::unordered_map<std::string, std::string> property_hashtable;
 	uint32_t _old_order_key;
-	
 }; /* class AxisView */
 
 #endif /* __ardour_gtk_axis_view_h__ */
