@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000-2007 Paul Davis 
+    Copyright (C) 2000-2007 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +22,8 @@
 #include <cfloat>
 #include <unistd.h>
 
-#include <ardour/curve.h>
+#include "ardour/automation_list.h"
+#include "evoral/Curve.hpp"
 
 using namespace std;
 using namespace ARDOUR;
@@ -33,7 +34,9 @@ curvetest (string filename)
 {
 	ifstream in (filename.c_str());
 	stringstream line;
-	Curve c (-1.0, +1.0, 0, true);
+	//Evoral::Parameter param(GainAutomation, -1.0, +1.0, 0.0);
+	Evoral::Parameter param(GainAutomation);
+	AutomationList al (param);
 	double minx = DBL_MAX;
 	double maxx = DBL_MIN;
 
@@ -42,7 +45,7 @@ curvetest (string filename)
 
 		in >> x;
 		in >> y;
-		
+
 		if (!in) {
 			break;
 		}
@@ -50,22 +53,22 @@ curvetest (string filename)
 		if (x < minx) {
 			minx = x;
 		}
-		
+
 		if (x > maxx) {
 			maxx = x;
 		}
-		
-		c.add (x, y);
+
+		al.add (x, y);
 	}
 
 
 	float foo[1024];
 
-	c.get_vector (minx, maxx, foo, 1024);
-	
+	al.curve().get_vector (minx, maxx, foo, 1024);
+
 	for (int i = 0; i < 1024; ++i) {
 	        cout << minx + (((double) i / 1024.0) * (maxx - minx)) << ' ' << foo[i] << endl;
 	}
-	
+
 	return 0;
 }
