@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000-2007 Paul Davis 
+    Copyright (C) 2000-2007 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include <string>
 #include <map>
-#include <ardour/types.h>
+#include "ardour/types.h"
 
 // This involves some cpp magic. --taybin
 
@@ -30,12 +30,16 @@
 #define SNAPMODE(a) /*empty*/
 #define REGIONLISTSORTTYPE(a) /*empty*/
 #define MOUSEMODE(a) /*empty*/
+#define MIDIEDITMODE(a) /*empty*/
 #define ZOOMFOCUS(a) /*empty*/
 #define DISPLAYCONTROL(a) /*empty*/
 #define IMPORTMODE(a) /*empty*/
-#define IMPORTPOSITION(a) /*empty*/
-#define IMPORTDISPOSITION(a) /*empty*/
+#define IMPORTPOSITION(a)
+#define IMPORTDISPOSITION(a)
 #define EDITPOINT(a) /*empty*/
+#define WAVEFORMSCALE(a) /*empty*/
+#define WAVEFORMSHAPE(a) /*empty*/
+#define INSERTTIMEOPT(a) /*empty*/
 
 namespace Editing {
 
@@ -95,6 +99,20 @@ MouseMode str2mousemode(const std::string &);
 #undef MOUSEMODE
 #define MOUSEMODE(a) /*empty*/
 
+// MIDIEDITMODE
+#undef MIDIEDITMODE
+#define MIDIEDITMODE(a) a,
+enum MidiEditMode {
+	#include "editing_syms.h"
+};
+
+extern const char *midieditmodestrs[];
+inline const char* enum2str(MidiEditMode m) {return midieditmodestrs[m];}
+MidiEditMode str2midieditmode(const std::string &);
+
+#undef MIDIEDITMODE
+#define MIDIEDITMODE(a) /*empty*/
+
 // ZOOMFOCUS
 #undef ZOOMFOCUS
 #define ZOOMFOCUS(a) a,
@@ -123,6 +141,7 @@ DisplayControl str2displaycontrol (const std::string &);
 #undef DISPLAYCONTROL
 #define DISPLAYCONTROL(a) /*empty*/
 
+
 // IMPORTMODE
 #undef IMPORTMODE
 #define IMPORTMODE(a) a,
@@ -163,9 +182,21 @@ enum EditPoint {
 #undef EDITPOINT
 #define EDITPOINT(a) /*empty*/
 
+// INSERTTIMEOPT
+#undef INSERTTIMEOPT
+#define INSERTTIMEOPT(a) a,
+enum InsertTimeOption {
+	#include "editing_syms.h"
+};
+
+#undef INSERTTIMEOPT
+#define INSERTTIMEOPT(a) /*empty*/
+
+
 /////////////////////
 // These don't need their state saved. yet...
 enum CutCopyOp {
+	Delete,
 	Cut,
 	Copy,
 	Clear
@@ -177,6 +208,13 @@ enum XFadeType {
 	At
 };
 
+enum EditIgnoreOption {
+	EDIT_IGNORE_NONE,
+	EDIT_IGNORE_PHEAD,
+	EDIT_IGNORE_MOUSE,
+	EDIT_IGNORE_MARKER
+};
+
 } // namespace Editing
 
 #endif // __gtk_ardour_editing_h__
