@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2001 Paul Davis 
+    Copyright (C) 2001 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,31 +17,41 @@
 
 */
 
+/* Note: public Editor methods are documented in public_editor.h */
+
+#include <inttypes.h>
 #include <unistd.h>
 #include <climits>
 
 #include <gtkmm/messagedialog.h>
 
-#include "export_session_dialog.h"
-#include "export_region_dialog.h"
-#include "export_range_markers_dialog.h"
+#include "pbd/gstdio_compat.h"
+
+#include "gtkmm2ext/choice.h"
+
+#include "pbd/pthread_utils.h"
+
+#include "ardour/audio_track.h"
+#include "ardour/audiofilesource.h"
+#include "ardour/audioplaylist.h"
+#include "ardour/audioregion.h"
+#include "ardour/chan_count.h"
+#include "ardour/midi_region.h"
+#include "ardour/session.h"
+#include "ardour/session_directory.h"
+#include "ardour/source_factory.h"
+#include "ardour/types.h"
+
+#include "audio_region_view.h"
+#include "audio_time_axis.h"
 #include "editor.h"
+#include "export_dialog.h"
+#include "midi_export_dialog.h"
+#include "midi_region_view.h"
 #include "public_editor.h"
 #include "selection.h"
 #include "time_axis_view.h"
-#include "audio_time_axis.h"
-#include "audio_region_view.h"
-
-#include <pbd/pthread_utils.h>
-#include <ardour/types.h>
-#include <ardour/export.h>
-#include <ardour/audio_track.h>
-#include <ardour/audiofilesource.h>
-#include <ardour/audio_diskstream.h>
-#include <ardour/audioregion.h>
-#include <ardour/audioplaylist.h>
-#include <ardour/source_factory.h>
-#include <ardour/audiofilesource.h>
+#include "utils.h"
 
 #include "i18n.h"
 
@@ -51,105 +61,156 @@ using namespace PBD;
 using namespace Gtk;
 
 void
-Editor::export_session()
+Editor::export_audio ()
 {
-	if (session) {
-		export_range (session->current_start_frame(), session->current_end_frame());
-	}
+	ExportDialog dialog (*this, _("Export"), ExportProfileManager::RegularExport);
+	dialog.set_session (_session);
+	dialog.run();
+}
+
+void
+Editor::stem_export ()
+{
+	StemExportDialog dialog (*this);
+	dialog.set_session (_session);
+	dialog.run();
 }
 
 void
 Editor::export_selection ()
 {
-	if (session) {
-		if (selection->time.empty()) {
-			MessageDialog message (*this, _("There is no selection to export.\n\nSelect a selection using the range mouse mode"));
-			message.run ();
-			return;
-		}
-
-		export_range (selection->time.front().start, selection->time.front().end);
-	}
+	ExportSelectionDialog dialog (*this);
+	dialog.set_session (_session);
+	dialog.run();
 }
 
 void
-Editor::export_range (nframes_t start, nframes_t end)
+Editor::export_range ()
 {
-	if (session) {
-		if (export_dialog == 0) {
-			export_dialog = new ExportSessionDialog (*this);
-		}
-		
-		export_dialog->connect_to_session (session);
-		export_dialog->set_range (start, end);
-		export_dialog->start_export();
-	}
-}	
+	ArdourMarker* marker;
 
+	if ((marker = reinterpret_cast<ArdourMarker *> (marker_menu_item->get_data ("marker"))) == 0) {
+		fatal << _("programming error: marker canvas item has no marker object pointer!") << endmsg;
+		abort(); /*NOTREACHED*/
+	}
+
+	Location* l;
+	bool is_start;
+
+	if (((l = find_location_from_marker (marker, is_start)) != 0) && (l->end() > l->start())) {
+		ExportRangeDialog dialog (*this, l->id().to_s());
+		dialog.set_session (_session);
+		dialog.run();
+	}
+}
+
+bool
+Editor::process_midi_export_dialog (MidiExportDialog& dialog, boost::shared_ptr<MidiRegion> midi_region)
+{
+	string path = dialog.get_path ();
+
+	if (Glib::file_test (path, Glib::FILE_TEST_EXISTS)) {
+		bool overwrite = ARDOUR_UI_UTILS::overwrite_file_dialog (dialog,
+									 _("Confirm MIDI File Overwrite"),
+									 _("A file with the same name already exists. Do you want to overwrite it?"));
+
+		if (!overwrite) {
+			return false;
+		}
+
+		/* force ::g_unlink because the backend code will
+		   go wrong if it tries to open an existing
+		   file for writing.
+		*/
+		::g_unlink (path.c_str());
+	}
+
+	(void) midi_region->clone (path);
+
+	return true;
+}
+
+/** Export the first selected region */
 void
 Editor::export_region ()
 {
-	if (clicked_regionview == 0) {
+	if (selection->regions.empty()) {
 		return;
 	}
 
-	ExportDialog* dialog = new ExportRegionDialog (*this, clicked_regionview->region());
-		
-	dialog->connect_to_session (session);
-	dialog->set_range (clicked_regionview->region()->first_frame(), clicked_regionview->region()->last_frame());
-	dialog->start_export();
-}
+	boost::shared_ptr<Region> r = selection->regions.front()->region();
+	boost::shared_ptr<AudioRegion> audio_region = boost::dynamic_pointer_cast<AudioRegion>(r);
+	boost::shared_ptr<MidiRegion> midi_region = boost::dynamic_pointer_cast<MidiRegion>(r);
 
-void
-Editor::export_range_markers ()
-{
-	if (session) {
+	if (audio_region) {
 
-		if (session->locations()->num_range_markers() == 0) {
-			MessageDialog message (*this, _("There are no ranges to export.\n\nCreate 1 or more ranges by dragging the mouse in the range bar"));
-			message.run ();
-			return;
+		RouteTimeAxisView & rtv (dynamic_cast<RouteTimeAxisView &> (selection->regions.front()->get_time_axis_view()));
+		AudioTrack & track (dynamic_cast<AudioTrack &> (*rtv.route()));
+
+		ExportRegionDialog dialog (*this, *(audio_region.get()), track);
+		dialog.set_session (_session);
+		dialog.run ();
+
+	} else if (midi_region) {
+
+		MidiExportDialog dialog (*this, midi_region);
+		dialog.set_session (_session);
+
+		bool finished = false;
+		while (!finished) {
+			switch (dialog.run ()) {
+			case Gtk::RESPONSE_ACCEPT:
+				finished = process_midi_export_dialog (dialog, midi_region);
+				break;
+			default:
+				finished = true;
+				return;
+			}
 		}
-		
-
-		if (export_range_markers_dialog == 0) {
-			export_range_markers_dialog = new ExportRangeMarkersDialog(*this);
-		}
-		
-		export_range_markers_dialog->connect_to_session (session);
-		export_range_markers_dialog->start_export();
 	}
-}	
+}
 
 int
 Editor::write_region_selection (RegionSelection& regions)
 {
 	for (RegionSelection::iterator i = regions.begin(); i != regions.end(); ++i) {
 		AudioRegionView* arv = dynamic_cast<AudioRegionView*>(*i);
-		if (arv)
+		if (arv) {
 			if (write_region ("", arv->audio_region()) == false)
 				return -1;
+		}
+
+		MidiRegionView* mrv = dynamic_cast<MidiRegionView*>(*i);
+		if (mrv) {
+			warning << "MIDI region export not implemented" << endmsg;
+		}
 	}
 
 	return 0;
 }
 
 void
-Editor::bounce_region_selection ()
+Editor::bounce_region_selection (bool with_processing)
 {
+	/* no need to check for bounceable() because this operation never puts
+	 * its results back in the playlist (only in the region list).
+	 */
+
 	for (RegionSelection::iterator i = selection->regions.begin(); i != selection->regions.end(); ++i) {
-		
+
 		boost::shared_ptr<Region> region ((*i)->region());
 		RouteTimeAxisView* rtv = dynamic_cast<RouteTimeAxisView*>(&(*i)->get_time_axis_view());
-		Track* track = dynamic_cast<Track*>(rtv->route().get());
+		boost::shared_ptr<Track> track = boost::dynamic_pointer_cast<Track> (rtv->route());
 
 		InterThreadInfo itt;
 
-		itt.done = false;
-		itt.cancel = false;
-		itt.progress = 0.0f;
+		boost::shared_ptr<Region> r;
 
-		track->bounce_range (region->position(), region->position() + region->length(), itt);
+		if (with_processing) {
+			r = track->bounce_range (region->position(), region->position() + region->length(), itt, track->main_outs(), false);
+		} else {
+			r = track->bounce_range (region->position(), region->position() + region->length(), itt, boost::shared_ptr<Processor>(), false);
+		}
 	}
 }
 
@@ -157,21 +218,23 @@ bool
 Editor::write_region (string path, boost::shared_ptr<AudioRegion> region)
 {
 	boost::shared_ptr<AudioFileSource> fs;
-	const nframes_t chunk_size = 4096;
-	nframes_t to_read;
+	const framepos_t chunk_size = 4096;
+	framepos_t to_read;
 	Sample buf[chunk_size];
 	gain_t gain_buffer[chunk_size];
-	nframes_t pos;
+	framepos_t pos;
 	char s[PATH_MAX+1];
 	uint32_t cnt;
 	vector<boost::shared_ptr<AudioFileSource> > sources;
 	uint32_t nchans;
-	
+
+	const string sound_directory = _session->session_directory().sound_path();
+
 	nchans = region->n_channels();
-	
+
 	/* don't do duplicate of the entire source if that's what is going on here */
 
-	if (region->start() == 0 && region->length() == region->source()->length()) {
+	if (region->start() == 0 && region->length() == region->source_length(0)) {
 		/* XXX should link(2) to create a new inode with "path" */
 		return true;
 	}
@@ -179,35 +242,38 @@ Editor::write_region (string path, boost::shared_ptr<AudioRegion> region)
 	if (path.length() == 0) {
 
 		for (uint32_t n=0; n < nchans; ++n) {
-			
+
 			for (cnt = 0; cnt < 999999; ++cnt) {
 				if (nchans == 1) {
-					snprintf (s, sizeof(s), "%s/%s_%" PRIu32 ".wav", session->sound_dir().c_str(),
+					snprintf (s, sizeof(s), "%s/%s_%" PRIu32 ".wav", sound_directory.c_str(),
 						  legalize_for_path(region->name()).c_str(), cnt);
 				}
 				else {
-					snprintf (s, sizeof(s), "%s/%s_%" PRIu32 "-%" PRId32 ".wav", session->sound_dir().c_str(),
+					snprintf (s, sizeof(s), "%s/%s_%" PRIu32 "-%" PRId32 ".wav", sound_directory.c_str(),
 						  legalize_for_path(region->name()).c_str(), cnt, n);
 				}
 
 				path = s;
-				
-				if (::access (path.c_str(), F_OK) != 0) {
+
+				if (!Glib::file_test (path, Glib::FILE_TEST_EXISTS)) {
 					break;
 				}
 			}
-			
+
 			if (cnt == 999999) {
 				error << "" << endmsg;
 				goto error_out;
 			}
-			
-		
-			
+
+
+
 			try {
-				fs = boost::dynamic_pointer_cast<AudioFileSource> (SourceFactory::createWritable (*session, path, false, session->frame_rate()));
+				fs = boost::dynamic_pointer_cast<AudioFileSource> (
+					SourceFactory::createWritable (DataType::AUDIO, *_session,
+					                               path, true,
+					                               false, _session->frame_rate()));
 			}
-			
+
 			catch (failed_constructor& err) {
 				goto error_out;
 			}
@@ -219,23 +285,23 @@ Editor::write_region (string path, boost::shared_ptr<AudioRegion> region)
 		/* TODO: make filesources based on passed path */
 
 	}
-	
+
 	to_read = region->length();
 	pos = region->position();
 
 	while (to_read) {
-		nframes_t this_time;
+		framepos_t this_time;
 
 		this_time = min (to_read, chunk_size);
 
 		for (vector<boost::shared_ptr<AudioFileSource> >::iterator src=sources.begin(); src != sources.end(); ++src) {
-			
+
 			fs = (*src);
 
 			if (region->read_at (buf, buf, gain_buffer, pos, this_time) != this_time) {
 				break;
 			}
-			
+
 			if (fs->write (buf, this_time) != this_time) {
 				error << "" << endmsg;
 				goto error_out;
@@ -250,7 +316,7 @@ Editor::write_region (string path, boost::shared_ptr<AudioRegion> region)
 	struct tm* now;
 	time (&tnow);
 	now = localtime (&tnow);
-	
+
 	for (vector<boost::shared_ptr<AudioFileSource> >::iterator src = sources.begin(); src != sources.end(); ++src) {
 		(*src)->update_header (0, *now, tnow);
 		(*src)->mark_immutable ();
@@ -286,9 +352,9 @@ Editor::write_audio_selection (TimeSelection& ts)
 
 		if (atv->is_audio_track()) {
 
-			boost::shared_ptr<AudioPlaylist> playlist = boost::dynamic_pointer_cast<AudioPlaylist>(atv->get_diskstream()->playlist());
-			
-			if (playlist && write_audio_range (*playlist, atv->get_diskstream()->n_channels(), ts) == 0) {
+			boost::shared_ptr<AudioPlaylist> playlist = boost::dynamic_pointer_cast<AudioPlaylist>(atv->track()->playlist());
+
+			if (playlist && write_audio_range (*playlist, atv->track()->n_channels(), ts) == 0) {
 				ret = -1;
 				break;
 			}
@@ -299,95 +365,102 @@ Editor::write_audio_selection (TimeSelection& ts)
 }
 
 bool
-Editor::write_audio_range (AudioPlaylist& playlist, uint32_t channels, list<AudioRange>& range)
+Editor::write_audio_range (AudioPlaylist& playlist, const ChanCount& count, list<AudioRange>& range)
 {
 	boost::shared_ptr<AudioFileSource> fs;
-	const nframes_t chunk_size = 4096;
-	nframes_t nframes;
+	const framepos_t chunk_size = 4096;
+	framepos_t nframes;
 	Sample buf[chunk_size];
 	gain_t gain_buffer[chunk_size];
-	nframes_t pos;
+	framepos_t pos;
 	char s[PATH_MAX+1];
 	uint32_t cnt;
 	string path;
 	vector<boost::shared_ptr<AudioFileSource> > sources;
 
+	const string sound_directory = _session->session_directory().sound_path();
+
+	uint32_t channels = count.n_audio();
+
 	for (uint32_t n=0; n < channels; ++n) {
-		
+
 		for (cnt = 0; cnt < 999999; ++cnt) {
 			if (channels == 1) {
-				snprintf (s, sizeof(s), "%s/%s_%" PRIu32 ".wav", session->sound_dir().c_str(),
+				snprintf (s, sizeof(s), "%s/%s_%" PRIu32 ".wav", sound_directory.c_str(),
 					  legalize_for_path(playlist.name()).c_str(), cnt);
 			}
 			else {
-				snprintf (s, sizeof(s), "%s/%s_%" PRIu32 "-%" PRId32 ".wav", session->sound_dir().c_str(),
+				snprintf (s, sizeof(s), "%s/%s_%" PRIu32 "-%" PRId32 ".wav", sound_directory.c_str(),
 					  legalize_for_path(playlist.name()).c_str(), cnt, n);
 			}
-			
-			if (::access (s, F_OK) != 0) {
+
+			if (!Glib::file_test (s, Glib::FILE_TEST_EXISTS)) {
 				break;
 			}
 		}
-		
+
 		if (cnt == 999999) {
 			error << "" << endmsg;
 			goto error_out;
 		}
 
 		path = s;
-		
+
 		try {
-			fs = boost::dynamic_pointer_cast<AudioFileSource> (SourceFactory::createWritable (*session, path, false, session->frame_rate()));
+			fs = boost::dynamic_pointer_cast<AudioFileSource> (
+				SourceFactory::createWritable (DataType::AUDIO, *_session,
+				                               path, true,
+				                               false, _session->frame_rate()));
 		}
-		
+
 		catch (failed_constructor& err) {
 			goto error_out;
 		}
-		
+
 		sources.push_back (fs);
 
 	}
-	
+
 
 	for (list<AudioRange>::iterator i = range.begin(); i != range.end();) {
-	
+
 		nframes = (*i).length();
 		pos = (*i).start;
-		
+
 		while (nframes) {
-			nframes_t this_time;
-			
+			framepos_t this_time;
+
 			this_time = min (nframes, chunk_size);
 
 			for (uint32_t n=0; n < channels; ++n) {
 
 				fs = sources[n];
-				
+
 				if (playlist.read (buf, buf, gain_buffer, pos, this_time, n) != this_time) {
 					break;
 				}
-				
+
 				if (fs->write (buf, this_time) != this_time) {
 					goto error_out;
 				}
 			}
-			
+
 			nframes -= this_time;
 			pos += this_time;
 		}
-		
+
 		list<AudioRange>::iterator tmp = i;
 		++tmp;
 
 		if (tmp != range.end()) {
-			
+
 			/* fill gaps with silence */
-			
+
 			nframes = (*tmp).start - (*i).end;
 
 			while (nframes) {
 
-				nframes_t this_time = min (nframes, chunk_size);
+				framepos_t this_time = min (nframes, chunk_size);
 				memset (buf, 0, sizeof (Sample) * this_time);
 
 				for (uint32_t n=0; n < channels; ++n) {
@@ -415,7 +488,7 @@ Editor::write_audio_range (AudioPlaylist& playlist, uint32_t channels, list<Audi
 		(*s)->mark_immutable ();
 		// do we need to ref it again?
 	}
-	
+
 	return true;
 
 error_out:
