@@ -1,6 +1,6 @@
 /*
     Copyright (C) 2001 Paul Davis
-    
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -18,10 +18,16 @@
 */
 
 #include <getopt.h>
+#include <string.h>
 #include <iostream>
 #include <cstdlib>
 
-#include <ardour/session.h>
+#include "ardour/debug.h"
+#include "ardour/session.h"
+
+#ifndef NDEBUG // "-H"
+#include "processor_box.h"
+#endif
 
 #include "opts.h"
 
@@ -29,41 +35,51 @@
 
 using namespace std;
 
-Glib::ustring ARDOUR_COMMAND_LINE::session_name = "";
-string ARDOUR_COMMAND_LINE::jack_client_name = "ardour";
+string ARDOUR_COMMAND_LINE::session_name = "";
+string ARDOUR_COMMAND_LINE::backend_client_name = "ardour";
+string ARDOUR_COMMAND_LINE::backend_session_uuid;
 bool  ARDOUR_COMMAND_LINE::show_key_actions = false;
-bool ARDOUR_COMMAND_LINE::no_splash = true;
+bool ARDOUR_COMMAND_LINE::no_splash = false;
 bool ARDOUR_COMMAND_LINE::just_version = false;
 bool ARDOUR_COMMAND_LINE::use_vst = true;
 bool ARDOUR_COMMAND_LINE::new_session = false;
 char* ARDOUR_COMMAND_LINE::curvetest_file = 0;
 bool ARDOUR_COMMAND_LINE::try_hw_optimization = true;
-Glib::ustring ARDOUR_COMMAND_LINE::keybindings_path = ""; /* empty means use builtin default */
-Glib::ustring ARDOUR_COMMAND_LINE::menus_file = "ardour.menus";
+bool ARDOUR_COMMAND_LINE::no_connect_ports = false;
+string ARDOUR_COMMAND_LINE::keybindings_path = ""; /* empty means use builtin default */
+std::string ARDOUR_COMMAND_LINE::menus_file = "ardour.menus";
 bool ARDOUR_COMMAND_LINE::finder_invoked_ardour = false;
+string ARDOUR_COMMAND_LINE::immediate_save;
+string ARDOUR_COMMAND_LINE::load_template;
+bool ARDOUR_COMMAND_LINE::check_announcements = true;
 
 using namespace ARDOUR_COMMAND_LINE;
 
 int
 print_help (const char *execname)
 {
-	cout << _("Usage: ") << execname << "\n"
-	     << _("  -v, --version                    Show version information\n")
-	     << _("  -h, --help                       Print this message\n")
-	     << _("  -b, --bindings                   Print all possible keyboard binding names\n")
-	     << _("  -c, --name  name                 Use a specific jack client name, default is ardour\n")
-	     << _("  -d, --disable-plugins            Disable all plugins in an existing session\n")
-	     << _("  -n, --show-splash                Show splash screen\n")
-	     << _("  -m, --menus file                 Use \"file\" for Ardour menus\n")                       
-	     << _("  -N, --new session-name           Create a new session from the command line\n")
-	     << _("  -O, --no-hw-optimizations        Disable h/w specific optimizations\n")
-	     << _("  -S, --sync	                      Draw the gui synchronously \n")
-#ifdef VST_SUPPORT
-	     << _("  -V, --novst                      Do not use VST support\n")
+	cout << _("Usage: ") << execname << " [OPTION]... [SESSION_NAME]\n\n"
+	     << _("  [SESSION_NAME]              Name of session to load\n")
+	     << _("  -v, --version               Show version information\n")
+	     << _("  -h, --help                  Print this message\n")
+	     << _("  -a, --no-announcements      Do not contact website for announcements\n")
+	     << _("  -b, --bindings              Print all possible keyboard binding names\n")
+	     << _("  -B, --bypass-plugins        Bypass all plugins in an existing session\n")
+	     << _("  -c, --name <name>           Use a specific backend client name, default is ardour\n")
+	     << _("  -d, --disable-plugins       Disable all plugins in an existing session\n")
+	     << _("  -D, --debug <options>       Set debug flags. Use \"-D list\" to see available options\n")
+	     << _("  -n, --no-splash             Do not show splash screen\n")
+	     << _("  -m, --menus file            Use \"file\" to define menus\n")
+	     << _("  -N, --new session-name      Create a new session from the command line\n")
+	     << _("  -O, --no-hw-optimizations   Disable h/w specific optimizations\n")
+	     << _("  -P, --no-connect-ports      Do not connect any ports at startup\n")
+	     << _("  -S, --sync                  Draw the gui synchronously \n")
+#ifdef WINDOWS_VST_SUPPORT
+	     << _("  -V, --novst                 Do not use VST support\n")
 #endif
-	     << _("  [session-name]                   Name of session to load\n")
-	     << _("  -C, --curvetest filename         Curve algorithm debugger\n")
-	     << _("  -k, --keybindings filename       Name of key bindings to load (default is ~/.ardour2/ardour.bindings)\n")
+	     << _("  -E, --save <file>           Load the specified session, save it to <file> and then quit\n")
+	     << _("  -C, --curvetest filename    Curve algorithm debugger\n")
+	     << _("  -k, --keybindings filename  Name of key bindings to load\n")
 		;
 	return 1;
 
@@ -72,7 +88,7 @@ print_help (const char *execname)
 int
 ARDOUR_COMMAND_LINE::parse_opts (int argc, char *argv[])
 {
-	const char *optstring = "U:hSbvVnOdc:C:m:N:k:p:";
+	const char *optstring = "abBc:C:dD:hHk:E:m:N:nOp:PST:U:vV";
 	const char *execname = strrchr (argv[0], '/');
 
 	if (getenv ("ARDOUR_SAE")) {
@@ -89,16 +105,23 @@ ARDOUR_COMMAND_LINE::parse_opts (int argc, char *argv[])
 	const struct option longopts[] = {
 		{ "version", 0, 0, 'v' },
 		{ "help", 0, 0, 'h' },
+		{ "no-announcements", 0, 0, 'a' },
 		{ "bindings", 0, 0, 'b' },
-		{ "show-splash", 0, 0, 'n' },
-                { "menus", 1, 0, 'm'} ,
+		{ "bypass-plugins", 0, 0, 'B' },
+		{ "disable-plugins", 0, 0, 'd' },
+		{ "debug", 1, 0, 'D' },
+		{ "no-splash", 0, 0, 'n' },
+		{ "menus", 1, 0, 'm' },
 		{ "name", 1, 0, 'c' },
 		{ "novst", 0, 0, 'V' },
 		{ "new", 1, 0, 'N' },
 		{ "no-hw-optimizations", 0, 0, 'O' },
 		{ "sync", 0, 0, 'S' },
 		{ "curvetest", 1, 0, 'C' },
-		{ "sillyAppleUndocumentedFinderFeature", 1, 0, 'p' },
+		{ "save", 1, 0, 'E' },
+		{ "uuid", 1, 0, 'U' },
+		{ "template", 1, 0, 'T' },
+		{ "no-connect-ports", 0, 0, 'P' },
 		{ 0, 0, 0, 0 }
 	};
 
@@ -115,7 +138,7 @@ ARDOUR_COMMAND_LINE::parse_opts (int argc, char *argv[])
 		switch (c) {
 		case 0:
 			break;
-		
+
 		case 'v':
 			just_version = true;
 			break;
@@ -124,30 +147,51 @@ ARDOUR_COMMAND_LINE::parse_opts (int argc, char *argv[])
 			print_help (execname);
 			exit (0);
 			break;
+		case 'H':
+#ifndef NDEBUG
+			ProcessorBox::show_all_processors = true;
+#endif
+			break;
+		case 'a':
+			check_announcements = false;
+			break;
 
 		case 'b':
 			show_key_actions = true;
 			break;
-			
+
+		case 'B':
+			ARDOUR::Session::set_bypass_all_loaded_plugins (true);
+			break;
+
 		case 'd':
 			ARDOUR::Session::set_disable_all_loaded_plugins (true);
 			break;
 
-                case 'm':
-                        menus_file = optarg;
-                        break;
+		case 'D':
+			if (PBD::parse_debug_options (optarg)) {
+				exit (0);
+			}
+			break;
+
+		case 'm':
+			menus_file = optarg;
+			break;
 
 		case 'n':
-			no_splash = false;
+			no_splash = true;
 			break;
 
 		case 'p':
 			//undocumented OS X finder -psn_XXXXX argument
 			finder_invoked_ardour = true;
 			break;
-		
+
 		case 'S':
-			// just pass this through to gtk it will figure it out
+		//	; just pass this through to gtk it will figure it out
+			break;
+		case 'T':
+			load_template = optarg;
 			break;
 
 		case 'N':
@@ -159,14 +203,18 @@ ARDOUR_COMMAND_LINE::parse_opts (int argc, char *argv[])
 			try_hw_optimization = false;
 			break;
 
+		case 'P':
+			no_connect_ports = true;
+			break;
+
 		case 'V':
-#ifdef VST_SUPPORT
+#ifdef WINDOWS_VST_SUPPORT
 			use_vst = false;
-#endif /* VST_SUPPORT */
+#endif /* WINDOWS_VST_SUPPORT */
 			break;
 
 		case 'c':
-			jack_client_name = optarg;
+			backend_client_name = optarg;
 			break;
 
 		case 'C':
@@ -177,6 +225,14 @@ ARDOUR_COMMAND_LINE::parse_opts (int argc, char *argv[])
 			keybindings_path = optarg;
 			break;
 
+		case 'E':
+			immediate_save = optarg;
+			break;
+
+		case 'U':
+			backend_session_uuid = optarg;
+                        break;
+
 		default:
 			return print_help(execname);
 		}
@@ -190,7 +246,6 @@ ARDOUR_COMMAND_LINE::parse_opts (int argc, char *argv[])
 		session_name = argv[optind++];
 	}
 
-
 	return 0;
 }
 
