@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000 Paul Davis 
+    Copyright (C) 2000 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,11 +23,16 @@
 #include <gtkmm/dialog.h>
 #include <gtkmm/notebook.h>
 #include <gtkmm/treeview.h>
+#include "gtkmm2ext/dndtreeview.h"
 #include <gtkmm2ext/selector.h>
 
-#include <ardour/plugin.h>
+#include "ardour/plugin.h"
+#include "ardour/session_handle.h"
 #include "plugin_interest.h"
 
+#include "ardour_button.h"
+#include "ardour_dialog.h"
+
 namespace ARDOUR {
 	class Session;
 	class PluginManager;
@@ -36,22 +41,21 @@ namespace ARDOUR {
 class PluginSelector : public ArdourDialog
 {
   public:
-	PluginSelector (ARDOUR::PluginManager *);
+	PluginSelector (ARDOUR::PluginManager&);
+	~PluginSelector ();
 
 	void set_interested_object (PluginInterestedObject&);
-	
+
 	int run (); // XXX should we try not to overload the non-virtual Gtk::Dialog::run() ?
 
-	void set_session (ARDOUR::Session*);
 	void on_show ();
 
-	Gtk::Menu& plugin_menu ();
+	Gtk::Menu* plugin_menu ();
 	void show_manager ();
 
   private:
 	PluginInterestedObject* interested_object;
 
-	ARDOUR::Session* session;
 	Gtk::ScrolledWindow scroller;   // Available plugins
 	Gtk::ScrolledWindow ascroller;  // Added plugins
 
@@ -59,33 +63,44 @@ class PluginSelector : public ArdourDialog
 	Gtk::Entry filter_entry;
 	Gtk::Button filter_button;
 
+	ArdourButton fil_hidden_button;
+	ArdourButton fil_instruments_button;
+	ArdourButton fil_analysis_button;
+	ArdourButton fil_utils_button;
+
 	void filter_button_clicked ();
 	void filter_entry_changed ();
 	void filter_mode_changed ();
-	
+
 	struct PluginColumns : public Gtk::TreeModel::ColumnRecord {
 		PluginColumns () {
 			add (favorite);
+			add (hidden);
 			add (name);
 			add (type_name);
 			add (category);
 			add (creator);
-			add (ins);
-			add (outs);
+			add (audio_ins);
+			add (audio_outs);
+			add (midi_ins);
+			add (midi_outs);
 			add (plugin);
 		}
 		Gtk::TreeModelColumn<bool> favorite;
+		Gtk::TreeModelColumn<bool> hidden;
 		Gtk::TreeModelColumn<std::string> name;
 		Gtk::TreeModelColumn<std::string> type_name;
 		Gtk::TreeModelColumn<std::string> category;
 		Gtk::TreeModelColumn<std::string> creator;
-		Gtk::TreeModelColumn<std::string> ins;
-		Gtk::TreeModelColumn<std::string> outs;
+		Gtk::TreeModelColumn<std::string> audio_ins;
+		Gtk::TreeModelColumn<std::string> audio_outs;
+		Gtk::TreeModelColumn<std::string> midi_ins;
+		Gtk::TreeModelColumn<std::string> midi_outs;
 		Gtk::TreeModelColumn<ARDOUR::PluginInfoPtr> plugin;
 	};
 	PluginColumns plugin_columns;
 	Glib::RefPtr<Gtk::ListStore> plugin_model;
-	Gtk::TreeView plugin_display;
+	Gtkmm2ext::DnDTreeView<ARDOUR::PluginInfoPtr> plugin_display;
 	Gtk::Button* btn_add;
 	Gtk::Button* btn_remove;
 
@@ -106,26 +121,44 @@ class PluginSelector : public ArdourDialog
 	void ladspa_refiller (const std::string&);
 	void lv2_refiller (const std::string&);
 	void vst_refiller (const std::string&);
+	void lxvst_refiller (const std::string&);
 	void au_refiller (const std::string&);
 
-	ARDOUR::PluginManager *manager;
+	Gtk::Menu* _plugin_menu;
+	ARDOUR::PluginManager& manager;
 
-	void row_clicked(GdkEventButton *);
+	void row_activated(Gtk::TreeModel::Path path, Gtk::TreeViewColumn* col);
 	void btn_add_clicked();
 	void btn_remove_clicked();
-	void btn_update_clicked();
 	void added_list_selection_changed();
+	void added_row_clicked(GdkEventButton* event);
 	void display_selection_changed();
 	void btn_apply_clicked();
 	ARDOUR::PluginPtr load_plugin (ARDOUR::PluginInfoPtr);
 	bool show_this_plugin (const ARDOUR::PluginInfoPtr&, const std::string&);
 	void setup_filter_string (std::string&);
 
-	void favorite_changed (const Glib::ustring& path);
+	bool fil_hidden_button_release (GdkEventButton*);
+	bool fil_instruments_button_release (GdkEventButton*);
+	bool fil_analysis_button_release (GdkEventButton*);
+	bool fil_utils_button_release (GdkEventButton*);
+
+	void favorite_changed (const std::string& path);
+	void hidden_changed (const std::string& path);
 	bool in_row_change;
 
 	void plugin_chosen_from_menu (const ARDOUR::PluginInfoPtr&);
-	Gtk::Menu* _menu;
+
+	Gtk::Menu* create_favs_menu (ARDOUR::PluginInfoList&);
+	Gtk::Menu* create_by_creator_menu (ARDOUR::PluginInfoList&);
+	Gtk::Menu* create_by_category_menu (ARDOUR::PluginInfoList&);
+	void build_plugin_menu ();
+	PBD::ScopedConnectionList plugin_list_changed_connection;
+
+	bool _show_hidden;
+	Gtkmm2ext::ActiveState _show_instruments;
+	Gtkmm2ext::ActiveState _show_analysers;
+	Gtkmm2ext::ActiveState _show_utils;
 };
 
 #endif // __ardour_plugin_selector_h__
