@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2004 Paul Davis 
+    Copyright (C) 2004 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@ class Selectable : public virtual sigc::trackable
 	virtual void set_selected (bool yn) {
 		if (yn != _selected) {
 			_selected = yn;
-			Selected (_selected); /* EMIT_SIGNAL */
 		}
 	}
 
@@ -42,9 +41,6 @@ class Selectable : public virtual sigc::trackable
 		return _selected;
 	}
 
-	/** Emitted when the selected status of this Selectable changes */
-	sigc::signal<void, bool> Selected ;
-
   protected:
 	bool _selected;
 };
