@@ -23,10 +23,10 @@
 #include <string>
 #include <vector>
 #include <map>
-#include <glibmm/ustring.h>
 
 #include <sigc++/signal.h>
 
+#include <gtkmm/stock.h>
 #include <gtkmm/box.h>
 #include <gtkmm/button.h>
 #include <gtkmm/checkbutton.h>
@@ -36,40 +36,51 @@
 #include <gtkmm/filechooserwidget.h>
 #include <gtkmm/frame.h>
 #include <gtkmm/label.h>
+#include <gtkmm/scale.h>
+#include <gtkmm/textview.h>
+#include <gtkmm/table.h>
+#include <gtkmm/liststore.h>
+#include <gtkmm/spinbutton.h>
+#include <gtkmm/notebook.h>
 
-#include <ardour/session.h>
-#include <ardour/audiofilesource.h>
 
-#include "ardour_dialog.h"
+#include "ardour/audiofilesource.h"
+#include "ardour/session_handle.h"
+
+#include "ardour_window.h"
 #include "editing.h"
+#include "audio_clock.h"
+#include "instrument_selector.h"
 
 namespace ARDOUR {
 	class Session;
 };
 
 class GainMeter;
+class Mootcher;
 
-class SoundFileBox : public Gtk::VBox
+class SoundFileBox : public Gtk::VBox, public ARDOUR::SessionHandlePtr, public PBD::ScopedConnectionList
 {
   public:
 	SoundFileBox (bool persistent);
 	virtual ~SoundFileBox () {};
-	
+
 	void set_session (ARDOUR::Session* s);
-	bool setup_labels (const Glib::ustring& filename);
+	bool setup_labels (const std::string& filename);
 
 	void audition();
 	bool audition_oneshot();
 	bool autoplay () const;
+	void set_src_quality(ARDOUR::SrcQuality q) { _src_quality = q; }
+	void set_import_position(Editing::ImportPosition p) { _import_position = p; }
 
   protected:
-	ARDOUR::Session* _session;
-	Glib::ustring path;
-	
+	std::string path;
+
 	ARDOUR::SoundFileInfo sf_info;
-	
+
 	Gtk::Table table;
-	
+
 	Gtk::Label length;
 	Gtk::Label format;
 	Gtk::Label channels;
@@ -78,7 +89,7 @@ class SoundFileBox : public Gtk::VBox
 
 	Gtk::Label channels_value;
 	Gtk::Label samplerate_value;
-	
+
 	Gtk::Label format_text;
 	AudioClock length_clock;
 	AudioClock timecode_clock;
@@ -87,62 +98,126 @@ class SoundFileBox : public Gtk::VBox
 	Gtk::Label preview_label;
 
 	Gtk::TextView tags_entry;
-	
+
 	Gtk::VBox main_box;
 	Gtk::VBox path_box;
 	Gtk::HBox bottom_box;
-	
+
 	Gtk::Button play_btn;
 	Gtk::Button stop_btn;
 	Gtk::CheckButton autoplay_btn;
 	Gtk::Button apply_btn;
+	Gtk::HScale seek_slider;
+
+	PBD::ScopedConnectionList auditioner_connections;
+	void audition_active(bool);
+	void audition_progress(ARDOUR::framecnt_t, ARDOUR::framecnt_t);
 
 	bool tags_entry_left (GdkEventFocus* event);
 	void tags_changed ();
 	void save_tags (const std::vector<std::string>&);
 	void stop_audition ();
+	bool seek_button_press(GdkEventButton*);
+	bool seek_button_release(GdkEventButton*);
+	bool _seeking;
+	ARDOUR::SrcQuality _src_quality;
+	Editing::ImportPosition _import_position;
 };
 
-class SoundFileBrowser : public ArdourDialog
+class SoundFileBrowser : public ArdourWindow
 {
   private:
 	class FoundTagColumns : public Gtk::TreeModel::ColumnRecord
 	{
 	  public:
-		Gtk::TreeModelColumn<Glib::ustring> pathname;
-		
+		Gtk::TreeModelColumn<std::string> pathname;
+
 		FoundTagColumns() { add(pathname); }
 	};
-	
+
+	class FreesoundColumns : public Gtk::TreeModel::ColumnRecord
+	{
+	  public:
+		Gtk::TreeModelColumn<std::string> id;
+		Gtk::TreeModelColumn<std::string> uri;
+		Gtk::TreeModelColumn<std::string> filename;
+		Gtk::TreeModelColumn<std::string> duration;
+		Gtk::TreeModelColumn<std::string> filesize;
+		Gtk::TreeModelColumn<std::string> smplrate;
+		Gtk::TreeModelColumn<std::string> license;
+		Gtk::TreeModelColumn<bool>        started;
+
+		FreesoundColumns() {
+			add(id);
+			add(filename);
+			add(uri);
+			add(duration);
+			add(filesize);
+			add(smplrate);
+			add(license);
+			add(started);
+		}
+	};
+
 	FoundTagColumns found_list_columns;
 	Glib::RefPtr<Gtk::ListStore> found_list;
 
+	FreesoundColumns freesound_list_columns;
+	Glib::RefPtr<Gtk::ListStore> freesound_list;
+
+	Gtk::Button freesound_more_btn;
+	Gtk::Button freesound_similar_btn;
+
+	void handle_freesound_results(std::string theString);
   public:
-	SoundFileBrowser (Gtk::Window& parent, std::string title, ARDOUR::Session* _s, bool persistent);
+	SoundFileBrowser (std::string title, ARDOUR::Session* _s, bool persistent);
 	virtual ~SoundFileBrowser ();
-	
+
+	int run ();
+	int status () const { return _status; }
+
 	virtual void set_session (ARDOUR::Session*);
-	std::vector<Glib::ustring> get_paths ();
-	
+	std::vector<std::string> get_paths ();
+
 	void clear_selection ();
 
 	Gtk::FileChooserWidget chooser;
-	Gtk::TreeView found_list_view;
 
-  protected:
-	bool resetting_ourselves;
-	
-	Gtk::FileFilter custom_filter;
-	Gtk::FileFilter matchall_filter;
 	SoundFileBox preview;
-	Gtk::HBox hpacker;
-
-	static Glib::ustring persistent_folder;
 
 	Gtk::Entry found_entry;
 	Gtk::Button found_search_btn;
+	Gtk::TreeView found_list_view;
+
+	Gtk::Entry freesound_entry;
+	Gtk::ComboBoxText freesound_sort;
+
+	Gtk::Button freesound_search_btn;
+	Gtk::TreeView freesound_list_view;
 	Gtk::Notebook notebook;
 
+	void freesound_search();
+	void refresh_display(std::string ID, std::string file);
+
+  protected:
+	bool resetting_ourselves;
+	int matches;
+	int _status;
+	bool _done;
+
+	Gtk::FileFilter audio_and_midi_filter;
+	Gtk::FileFilter audio_filter;
+	Gtk::FileFilter midi_filter;
+	Gtk::FileFilter custom_filter;
+	Gtk::FileFilter matchall_filter;
+	Gtk::HBox hpacker;
+	Gtk::VBox vpacker;
+
+	Gtk::Button import_button;
+	Gtk::Button close_button;
+
+	static std::string persistent_folder;
+
 	GainMeter* gm;
 	Gtk::VBox meter_packer;
 	void add_gain_meter ();
@@ -153,28 +228,41 @@ class SoundFileBrowser : public ArdourDialog
 	sigc::connection metering_connection;
 
 	void update_preview ();
+
 	void found_list_view_selected ();
 	void found_list_view_activated (const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn*);
 	void found_search_clicked ();
 
-	void chooser_file_activated ();
-	
-	bool on_custom (const Gtk::FileFilter::Info& filter_info);
+	void freesound_list_view_selected ();
+	void freesound_list_view_activated (const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn*);
+	void freesound_search_clicked ();
+	void freesound_more_clicked ();
+	void freesound_similar_clicked ();
+	int freesound_page;
 
-	virtual bool reset_options() { return true; }
+	void chooser_file_activated ();
+	std::string freesound_get_audio_file(Gtk::TreeIter iter);
+
+	bool on_audio_filter (const Gtk::FileFilter::Info& filter_info);
+	bool on_midi_filter (const Gtk::FileFilter::Info& filter_info);
+	bool on_audio_and_midi_filter (const Gtk::FileFilter::Info& filter_info);
+
+	void set_action_sensitive (bool);
+
+	virtual bool reset_options () { return true; }
 
   protected:
 	void on_show();
-
+	virtual void do_something(int action);
 };
 
 class SoundFileChooser : public SoundFileBrowser
 {
   public:
-	SoundFileChooser (Gtk::Window& parent, std::string title, ARDOUR::Session* _s = 0);
+	SoundFileChooser (std::string title, ARDOUR::Session* _s = 0);
 	virtual ~SoundFileChooser () {};
-	
-	Glib::ustring get_filename ();
+
+	std::string get_filename ();
 
   protected:
 	void on_hide();
@@ -184,15 +272,18 @@ class SoundFileOmega : public SoundFileBrowser
 {
 
   public:
-	SoundFileOmega (Gtk::Window& parent, std::string title, ARDOUR::Session* _s, int selected_tracks, bool persistent, 
+	SoundFileOmega (std::string title, ARDOUR::Session* _s,
+			uint32_t selected_audio_tracks, uint32_t selected_midi_tracks,
+			bool persistent,
 			Editing::ImportMode mode_hint = Editing::ImportAsTrack);
-	
-	void reset (int selected_tracks);
-	
+
+	void reset (uint32_t selected_audio_tracks, uint32_t selected_midi_tracks);
+
 	Gtk::ComboBoxText action_combo;
 	Gtk::ComboBoxText where_combo;
 	Gtk::ComboBoxText channel_combo;
 	Gtk::ComboBoxText src_combo;
+	InstrumentSelector instrument_combo;
 
 	Gtk::CheckButton copy_files_btn;
 
@@ -206,9 +297,10 @@ class SoundFileOmega : public SoundFileBrowser
 	void on_hide();
 
   private:
-	uint32_t selected_track_cnt;
+	uint32_t selected_audio_track_cnt;
+	uint32_t selected_midi_track_cnt;
 
-	typedef std::map<Glib::ustring,Editing::ImportDisposition> DispositionMap;
+	typedef std::map<std::string,Editing::ImportDisposition> DispositionMap;
 	DispositionMap disposition_map;
 
 	Gtk::HBox options;
@@ -216,15 +308,22 @@ class SoundFileOmega : public SoundFileBrowser
 	Gtk::VBox block_three;
 	Gtk::VBox block_four;
 
-	bool check_info (const std::vector<Glib::ustring>& paths, 
+	bool check_info (const std::vector<std::string>& paths,
 			 bool& same_size, bool& src_needed, bool& multichannel);
 
-	static bool check_link_status (const ARDOUR::Session&, const std::vector<Glib::ustring>& paths);
+	static bool check_link_status (const ARDOUR::Session*, const std::vector<std::string>& paths);
 
 	void file_selection_changed ();
 	bool reset_options ();
 	void reset_options_noret ();
 	bool bad_file_message ();
+	void src_combo_changed ();
+	void where_combo_changed ();
+
+	void do_something (int action);
+
+	bool _import_active;
+	bool _reset_post_import;
 };
 
 #endif // __ardour_sfdb_ui_h__
