@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000-2007 Paul Davis 
+    Copyright (C) 2000-2007 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,73 +30,77 @@
 #include <gtkmm/spinbutton.h>
 #include <gtkmm/comboboxtext.h>
 
-#include <ardour/types.h>
-#include <ardour/tempo.h>
+#include "ardour/types.h"
+#include "ardour/tempo.h"
 
 #include "ardour_dialog.h"
 
-struct TempoDialog : public ArdourDialog 
+class TempoDialog : public ArdourDialog
 {
-	Gtk::ComboBoxText note_types;
-	vector<string> strings;
-	Gtk::Adjustment   bpm_adjustment;
-	Gtk::SpinButton   bpm_spinner;
-	Gtk::Frame        bpm_frame;
-	Gtk::Button  ok_button;
-	Gtk::Button  cancel_button;
-	Gtk::HBox    hspacer1, hspacer2;
-	Gtk::VBox    vspacer1;
-	Gtk::Entry   when_bar_entry;
-	Gtk::Entry   when_beat_entry;
-	Gtk::Label   when_bar_label;
-	Gtk::Label   when_beat_label;
-	Gtk::Table   when_table;
-	Gtk::Frame   when_frame;
-	char buf[64];
-    
-	TempoDialog (ARDOUR::TempoMap&, nframes_t, const string & action);
-	TempoDialog (ARDOUR::TempoSection&, const string & action);
+public:
+	TempoDialog (ARDOUR::TempoMap&, framepos_t, const std::string & action);
+	TempoDialog (ARDOUR::TempoSection&, const std::string & action);
 
 	double get_bpm ();
 	double get_note_type ();
-	bool   get_bbt_time (ARDOUR::BBT_Time&);
-    
+	bool   get_bbt_time (Timecode::BBT_Time&);
+
 private:
-	void init (const ARDOUR::BBT_Time& start, double, double, bool);
+	void init (const Timecode::BBT_Time& start, double, double, bool);
+	bool is_user_input_valid() const;
 	void bpm_changed ();
 	bool bpm_button_press (GdkEventButton* );
 	bool bpm_button_release (GdkEventButton* );
-	void note_types_change ();
-};
+	bool entry_key_release (GdkEventKey* );
+	void pulse_change ();
+	bool tap_tempo_button_press (GdkEventButton* );
+	bool tap_tempo_focus_out (GdkEventFocus* );
 
-struct MeterDialog : public ArdourDialog 
-{
-	Gtk::Entry   bpb_entry;
-	Gtk::ComboBoxText note_types;
-	vector<string> strings;
-	Gtk::Frame   bpb_frame;
-	Gtk::Button  ok_button;
-	Gtk::Button  cancel_button;
+	typedef std::map<std::string,float> NoteTypes;
+	NoteTypes note_types;
+
+	bool tapped;      // whether the tap-tempo button has been clicked
+	gint64 last_tap;
+	double average_interval;
+
+	Gtk::ComboBoxText pulse_selector;
+	Gtk::Adjustment   bpm_adjustment;
+	Gtk::SpinButton   bpm_spinner;
 	Gtk::Entry   when_bar_entry;
 	Gtk::Entry   when_beat_entry;
 	Gtk::Label   when_bar_label;
 	Gtk::Label   when_beat_label;
-	Gtk::Table   when_table;
-	Gtk::Frame   when_frame;
-	char buf[64];
-    
-	MeterDialog (ARDOUR::TempoMap&, nframes_t, const string & action);
-	MeterDialog (ARDOUR::MeterSection&, const string & action);
+	Gtk::Label   pulse_selector_label;
+	Gtk::Button  tap_tempo_button;
+};
+
+class MeterDialog : public ArdourDialog
+{
+public:
+
+	MeterDialog (ARDOUR::TempoMap&, framepos_t, const std::string & action);
+	MeterDialog (ARDOUR::MeterSection&, const std::string & action);
 
 	double get_bpb ();
 	double get_note_type ();
-	bool   get_bbt_time (ARDOUR::BBT_Time&);
+	bool   get_bbt_time (Timecode::BBT_Time&);
 
 private:
-	void init (const ARDOUR::BBT_Time&, double, double, bool);
-	bool bpb_key_press (GdkEventKey* );
-	bool bpb_key_release (GdkEventKey* );
-	void note_types_change ();
+	void init (const Timecode::BBT_Time&, double, double, bool);
+	bool is_user_input_valid() const;
+	bool entry_key_press (GdkEventKey* );
+	bool entry_key_release (GdkEventKey* );
+	void note_type_change ();
+
+	typedef std::map<std::string,float> NoteTypes;
+	NoteTypes note_types;
+
+	Gtk::Entry   bpb_entry;
+	Gtk::ComboBoxText note_type;
+	std::vector<std::string> strings;
+	Gtk::Button  ok_button;
+	Gtk::Button  cancel_button;
+	Gtk::Entry   when_bar_entry;
 };
 
 #endif /* __ardour_gtk_tempo_dialog_h__ */
