@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2006 Paul Davis 
+    Copyright (C) 2006 Paul Davis
     Written by Taybin Rutkin
 
     This program is free software; you can redistribute it and/or modify
@@ -30,11 +30,11 @@
 #include <vector>
 #include <map>
 
-#include <ardour/plugin.h>
+#include "ardour/plugin.h"
 
 #include <AudioUnit/AudioUnit.h>
 #include <AudioUnit/AudioUnitProperties.h>
-#include <appleutility/AUParamInfo.h>
+#include "AUParamInfo.h"
 
 #include <boost/shared_ptr.hpp>
 
@@ -48,77 +48,83 @@ namespace ARDOUR {
 class AudioEngine;
 class Session;
 
-struct AUParameterDescriptor : public Plugin::ParameterDescriptor {
+struct LIBARDOUR_API AUParameterDescriptor : public ParameterDescriptor {
 	// additional fields to make operations more efficient
 	AudioUnitParameterID id;
 	AudioUnitScope scope;
 	AudioUnitElement element;
-	float default_value;
 	bool automatable;
-	AudioUnitParameterUnit unit;
+	AudioUnitParameterUnit au_unit;
 };
 
-class AUPlugin : public ARDOUR::Plugin
+class LIBARDOUR_API AUPlugin : public ARDOUR::Plugin
 {
   public:
 	AUPlugin (AudioEngine& engine, Session& session, boost::shared_ptr<CAComponent> comp);
 	AUPlugin (const AUPlugin& other);
 	virtual ~AUPlugin ();
-	
-        std::string unique_id () const;
+
+	std::string unique_id () const;
 	const char * label () const;
 	const char * name () const { return _info->name.c_str(); }
 	const char * maker () const { return _info->creator.c_str(); }
 	uint32_t parameter_count () const;
 	float default_value (uint32_t port);
-	nframes_t latency () const;
+	framecnt_t signal_latency() const;
 	void set_parameter (uint32_t which, float val);
 	float get_parameter (uint32_t which) const;
-    
+
 	int get_parameter_descriptor (uint32_t which, ParameterDescriptor&) const;
 	uint32_t nth_parameter (uint32_t which, bool& ok) const;
 	void activate ();
 	void deactivate ();
-	void set_block_size (nframes_t nframes);
-    
-	int connect_and_run (vector<Sample*>& bufs, uint32_t maxbuf, int32_t& in, int32_t& out, nframes_t nframes, nframes_t offset);
-	std::set<uint32_t> automatable() const;
-	string describe_parameter (uint32_t);
-	string state_node_name () const { return "audiounit"; }
+	void flush ();
+	int set_block_size (pframes_t nframes);
+
+	int connect_and_run (BufferSet& bufs,
+			     ChanMapping in, ChanMapping out,
+			     pframes_t nframes, framecnt_t offset);
+	std::set<Evoral::Parameter> automatable() const;
+	std::string describe_parameter (Evoral::Parameter);
+	std::string state_node_name () const { return "audiounit"; }
 	void print_parameter (uint32_t, char*, uint32_t len) const;
-    
+
 	bool parameter_is_audio (uint32_t) const;
 	bool parameter_is_control (uint32_t) const;
 	bool parameter_is_input (uint32_t) const;
 	bool parameter_is_output (uint32_t) const;
-    
-	XMLNode& get_state();
-	int set_state(const XMLNode& node);
-	
-	bool save_preset (string name);
-	bool load_preset (const string preset_label);
-	std::vector<std::string> get_presets ();
+
+	void set_info (PluginInfoPtr);
+
+	int set_state(const XMLNode& node, int);
+
+	bool load_preset (PresetRecord);
 	std::string current_preset() const;
 
 	bool has_editor () const;
-	
-	int32_t can_do (int32_t in, int32_t& out);
-	uint32_t output_streams() const;
-	uint32_t input_streams() const;
-	int32_t configure_io (int32_t in, int32_t out);
+
+	bool can_support_io_configuration (const ChanCount& in, ChanCount& out);
+	ChanCount output_streams() const;
+	ChanCount input_streams() const;
+	bool configure_io (ChanCount in, ChanCount out);
+	bool requires_fixed_size_buffers() const;
+
+	void set_fixed_size_buffers (bool yn) {
+		_requires_fixed_size_buffers = yn;
+	}
 
 	boost::shared_ptr<CAAudioUnit> get_au () { return unit; }
 	boost::shared_ptr<CAComponent> get_comp () const { return comp; }
-    
-        OSStatus render_callback(AudioUnitRenderActionFlags *ioActionFlags,
-				 const AudioTimeStamp    *inTimeStamp,
-				 UInt32       inBusNumber,
-				 UInt32       inNumberFrames,
-				 AudioBufferList*       ioData);
+
+	OSStatus render_callback(AudioUnitRenderActionFlags *ioActionFlags,
+	                         const AudioTimeStamp       *inTimeStamp,
+	                         UInt32                      inBusNumber,
+	                         UInt32                      inNumberFrames,
+	                         AudioBufferList*            ioData);
 
 	/* "host" callbacks */
 
-	OSStatus get_beat_and_tempo_callback (Float64* outCurrentBeat, 
+	OSStatus get_beat_and_tempo_callback (Float64* outCurrentBeat,
 					      Float64* outCurrentTempo);
 
 	OSStatus get_musical_time_location_callback (UInt32*  outDeltaSampleOffsetToNextBeat,
@@ -133,15 +139,39 @@ class AUPlugin : public ARDOUR::Plugin
 					       Float64*  outCycleStartBeat,
 					       Float64*  outCycleEndBeat);
 
+	static std::string maybe_fix_broken_au_id (const std::string&);
+
+        /* this MUST be called from thread in which you want to receive notifications
+	   about parameter changes.
+	*/
+	int create_parameter_listener (AUEventListenerProc callback, void *arg, float interval_secs);
+        /* these can be called from any thread but SHOULD be called from the same thread
+	   that will receive parameter change notifications.
+	*/
+	int listen_to_parameter (uint32_t param_id);
+	int end_listen_to_parameter (uint32_t param_id);
+
+
+  protected:
+	std::string do_save_preset (std::string name);
+	void do_remove_preset (std::string);
+
   private:
-        boost::shared_ptr<CAComponent> comp;
-        boost::shared_ptr<CAAudioUnit> unit;
-	
-        bool initialized;
+	void find_presets ();
+
+	boost::shared_ptr<CAComponent> comp;
+	boost::shared_ptr<CAAudioUnit> unit;
+
+	bool initialized;
 	int32_t input_channels;
 	int32_t output_channels;
 	std::vector<std::pair<int,int> > io_configs;
+	pframes_t _current_block_size;
+	framecnt_t _last_nframes;
+	bool _requires_fixed_size_buffers;
 	AudioBufferList* buffers;
+	bool _has_midi_input;
+	bool _has_midi_output;
 
 	/* despite all the cool work that apple did on their AU preset
 	   system, they left factory presets and user presets as two
@@ -159,66 +189,90 @@ class AUPlugin : public ARDOUR::Plugin
 	UInt32 global_elements;
 	UInt32 output_elements;
 	UInt32 input_elements;
-	
+
 	int set_output_format (AudioStreamBasicDescription&);
 	int set_input_format (AudioStreamBasicDescription&);
 	int set_stream_format (int scope, uint32_t cnt, AudioStreamBasicDescription&);
-        int _set_block_size (nframes_t nframes);
 	void discover_parameters ();
+	void add_state (XMLNode *) const;
+
+	typedef std::map<uint32_t, uint32_t> ParameterMap;
+	ParameterMap parameter_map;
+	uint32_t   input_maxbuf;
+	framecnt_t input_offset;
+	framecnt_t cb_offset;
+	BufferSet* input_buffers;
+	framecnt_t frames_processed;
+	uint32_t   audio_input_cnt;
 
-	std::vector<std::pair<uint32_t, uint32_t> > parameter_map;
-	uint32_t current_maxbuf;
-        nframes_t current_offset;
-        nframes_t cb_offset;
-        vector<Sample*>* current_buffers;
-        nframes_t frames_processed;
-	
 	std::vector<AUParameterDescriptor> descriptors;
+	AUEventListenerRef _parameter_listener;
+	void * _parameter_listener_arg;
 	void init ();
 
 	void discover_factory_presets ();
 
 	bool      last_transport_rolling;
 	float     last_transport_speed;
+
+	static void _parameter_change_listener (void* /*arg*/, void* /*src*/, const AudioUnitEvent* event, UInt64 host_time, Float32 new_value);
+	void parameter_change_listener (void* /*arg*/, void* /*src*/, const AudioUnitEvent* event, UInt64 host_time, Float32 new_value);
 };
-	
+
 typedef boost::shared_ptr<AUPlugin> AUPluginPtr;
 
-struct AUPluginCachedInfo { 
+struct LIBARDOUR_API AUPluginCachedInfo {
 	std::vector<std::pair<int,int> > io_configs;
 };
 
-class AUPluginInfo : public PluginInfo {
-  public:	
+class LIBARDOUR_API AUPluginInfo : public PluginInfo {
+  public:
 	 AUPluginInfo (boost::shared_ptr<CAComponentDescription>);
 	~AUPluginInfo ();
 
 	PluginPtr load (Session& session);
 
+	std::vector<Plugin::PresetRecord> get_presets (bool user_only) const;
+
+	bool needs_midi_input ();
+	bool is_effect () const;
+	bool is_effect_without_midi_input () const;
+	bool is_effect_with_midi_input () const;
+	bool is_instrument () const;
+
 	AUPluginCachedInfo cache;
 
-	static PluginInfoList discover ();
-	static void get_names (CAComponentDescription&, std::string& name, Glib::ustring& maker);
-        static std::string stringify_descriptor (const CAComponentDescription&);
+	bool reconfigurable_io() const { return true; }
+
+	static PluginInfoList* discover (bool scan_only);
+	static bool au_get_crashlog (std::string &msg);
+	static std::string stringify_descriptor (const CAComponentDescription&);
 
 	static int load_cached_info ();
 
   private:
 	boost::shared_ptr<CAComponentDescription> descriptor;
 	UInt32 version;
-	
+	static FILE * _crashlog_fd;
+	static bool _scan_only;
+
+	static void au_start_crashlog (void);
+	static void au_remove_crashlog (void);
+	static void au_crashlog (std::string);
+
 	static void discover_music (PluginInfoList&);
 	static void discover_fx (PluginInfoList&);
 	static void discover_generators (PluginInfoList&);
+	static void discover_instruments (PluginInfoList&);
 	static void discover_by_description (PluginInfoList&, CAComponentDescription&);
 	static Glib::ustring au_cache_path ();
 
 	typedef std::map<std::string,AUPluginCachedInfo> CachedInfoMap;
 	static CachedInfoMap cached_info;
-	
-	static bool cached_io_configuration (const std::string&, UInt32, CAComponent&, AUPluginCachedInfo&, const std::string& name);
+
+	static int cached_io_configuration (const std::string&, UInt32, CAComponent&, AUPluginCachedInfo&, const std::string& name);
 	static void add_cached_info (const std::string&, AUPluginCachedInfo&);
-	static void save_cached_info ();	
+	static void save_cached_info ();
 };
 
 typedef boost::shared_ptr<AUPluginInfo> AUPluginInfoPtr;
