@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2001 Paul Davis 
+    Copyright (C) 2001 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,20 +15,36 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: dB.h,v 1.3 2005/04/18 19:08:05 pauld Exp $
 */
 
 #ifndef __ardour_dB_h__
 #define __ardour_dB_h__
 
-#include <pbd/fastlog.h>
+#include <limits>
+#include "pbd/fastlog.h"
+
+#define GAIN_COEFF_ZERO 0.f
+#define GAIN_COEFF_SMALL 0.0000001f  //-140dB
+#define GAIN_COEFF_UNITY 1.f
 
 static inline float dB_to_coefficient (float dB) {
 	return dB > -318.8f ? pow (10.0f, dB * 0.05f) : 0.0f;
 }
 
-static inline float coefficient_to_dB (float coeff) {
+static inline float fast_coefficient_to_dB (float coeff) {
 	return 20.0f * fast_log10 (coeff);
 }
 
+static inline float accurate_coefficient_to_dB (float coeff) {
+	if (coeff < 1e-15) return  -std::numeric_limits<float>::infinity();
+	return 20.0f * log10f (coeff);
+}
+
+static inline double dB_coeff_step(double max_coeff) {
+	const double max_db = lrint(accurate_coefficient_to_dB(max_coeff));
+	return 0.1 * (max_coeff / max_db);
+}
+
+extern double zero_db_as_fraction;
+
 #endif /* __ardour_dB_h__ */
