@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2007 Paul Davis 
+    Copyright (C) 2007 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,22 +20,26 @@
 #ifndef __ardour_importable_source_h__
 #define __ardour_importable_source_h__
 
-#include <pbd/failed_constructor.h>
-#include <ardour/types.h>
+#include "pbd/failed_constructor.h"
+#include "ardour/libardour_visibility.h"
+#include "ardour/types.h"
 
 namespace ARDOUR {
 
-class ImportableSource {
+class LIBARDOUR_API ImportableSource {
 public:
 	ImportableSource () {}
 	virtual ~ImportableSource() {}
 
-	virtual nframes_t read (Sample* buffer, nframes_t nframes) = 0;
-	virtual float ratio() const { return 1.0f; }
-	virtual uint32_t channels() const = 0;
-	virtual nframes_t length() const = 0;
-	virtual nframes_t samplerate() const = 0;
-	virtual void      seek (nframes_t pos) = 0;
+	virtual framecnt_t read (Sample* buffer, framecnt_t nframes) = 0;
+	virtual float      ratio() const { return 1.0f; }
+	virtual uint32_t   channels() const = 0;
+	virtual framecnt_t length() const = 0;
+	virtual framecnt_t samplerate() const = 0;
+	virtual void       seek (framepos_t pos) = 0;
+	virtual framepos_t natural_position() const = 0;
+
+	virtual bool clamped_at_unity () const = 0;
 };
 
 }
