@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2007 Paul Davis 
+    Copyright (C) 2007 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,18 +20,19 @@
 #ifndef __ardour_rbeffect_h__
 #define __ardour_rbeffect_h__
 
-#include <ardour/audiofilter.h>
+#include "ardour/filter.h"
+#include "ardour/timefx_request.h"
 
 namespace ARDOUR {
 
 class AudioRegion;
 
-class RBEffect : public AudioFilter {
+class LIBARDOUR_API RBEffect : public Filter {
   public:
 	RBEffect (ARDOUR::Session&, TimeFXRequest&);
 	~RBEffect ();
 
-	int run (boost::shared_ptr<ARDOUR::AudioRegion>);
+	int run (boost::shared_ptr<ARDOUR::Region>, Progress* progress = 0);
 
   private:
 	TimeFXRequest& tsr;
