@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2000-2007 Paul Davis 
+    Copyright (C) 2000-2007 Paul Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,17 +26,19 @@
 #include <gtkmm/treeselection.h>
 #include <gtkmm/selectiondata.h>
 
+#include "gtkmm2ext/visibility.h"
+
 namespace Gtkmm2ext {
 
 template<class DataType>
-struct SerializedObjectPointers {
+struct /*LIBGTKMM2EXT_API*/ SerializedObjectPointers {
     uint32_t size;
     uint32_t cnt;
     char     type[32];
     DataType data[0];
 };
 
-class DnDTreeViewBase : public Gtk::TreeView 
+class LIBGTKMM2EXT_API DnDTreeViewBase : public Gtk::TreeView
 {
   private:
   public:
@@ -45,7 +47,16 @@ class DnDTreeViewBase : public Gtk::TreeView
 
 	void add_drop_targets (std::list<Gtk::TargetEntry>&);
 	void add_object_drag (int column, std::string type_name);
-	
+
+	void on_drag_begin (Glib::RefPtr<Gdk::DragContext> const & context);
+	void on_drag_end (Glib::RefPtr<Gdk::DragContext> const & context);
+
+	bool on_button_press_event (GdkEventButton *ev) {
+		press_start_x = ev->x;
+		press_start_y = ev->y;
+		return TreeView::on_button_press_event (ev);
+	}
+
 	void on_drag_leave(const Glib::RefPtr<Gdk::DragContext>& context, guint time) {
 		suggested_action = context->get_suggested_action();
 		TreeView::on_drag_leave (context, time);
@@ -58,31 +69,64 @@ class DnDTreeViewBase : public Gtk::TreeView
 
 	bool on_drag_drop(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, guint time);
 
+	void set_drag_column (int c) {
+		_drag_column = c;
+	}
+
   protected:
 	std::list<Gtk::TargetEntry> draggable;
 	Gdk::DragAction             suggested_action;
 	int                         data_column;
+	std::string                 object_type;
+
+	double press_start_x;
+	double press_start_y;
+	int _drag_column;
+
+	struct DragData {
+	    DragData () : source (0) {}
+
+	    Gtk::TreeView* source;
+	    int            data_column;
+	    std::string    object_type;
+	};
+
+	static DragData drag_data;
+
+	void start_object_drag () {
+		drag_data.source = this;
+		drag_data.data_column = data_column;
+		drag_data.object_type = object_type;
+	}
+
+	void end_object_drag () {
+		drag_data.source = 0;
+		drag_data.data_column = -1;
+		drag_data.object_type = "";
+	}
 };
 
 template<class DataType>
-class DnDTreeView : public DnDTreeViewBase
+class /*LIBGTKMM2EXT_API*/ DnDTreeView : public DnDTreeViewBase
 {
   public:
-	DnDTreeView() {} 
+	DnDTreeView() {}
 	~DnDTreeView() {}
 
-	sigc::signal<void,std::string,uint32_t,const DataType*> signal_object_drop;
+	sigc::signal<void, const Glib::RefPtr<Gdk::DragContext>&, const Gtk::SelectionData&> signal_drop;
 
 	void on_drag_data_get(const Glib::RefPtr<Gdk::DragContext>& context, Gtk::SelectionData& selection_data, guint info, guint time) {
 		if (selection_data.get_target() == "GTK_TREE_MODEL_ROW") {
-			
+
 			TreeView::on_drag_data_get (context, selection_data, info, time);
-			
-		} else if (data_column >= 0) {
-			
-			Gtk::TreeSelection::ListHandle_Path selection = get_selection()->get_selected_rows ();
-			SerializedObjectPointers<DataType>* sr = serialize_pointers (get_model(), &selection, selection_data.get_target());
-			selection_data.set (8, (guchar*)sr, sr->size);
+
+		} else if (selection_data.get_target() == object_type) {
+
+			/* return a pointer to this object, which allows
+			 * the receiver to call on_drag_data_received()
+			 */
+			void *c = this;
+			selection_data.set (8, (guchar*)&c, sizeof(void*));
 		}
 	}
 
@@ -95,71 +139,41 @@ class DnDTreeView : public DnDTreeViewBase
 			TreeView::on_drag_data_received (context, x, y, selection_data, info, time);
 			return;
 		}
-		
-		if (selection_data.get_target() == "GTK_TREE_MODEL_ROW") {
-			
-			TreeView::on_drag_data_received (context, x, y, selection_data, info, time);
 
-		} else if (data_column >= 0) {
-			
-			/* object D-n-D */
-			
-			const void* data = selection_data.get_data();
-			const SerializedObjectPointers<DataType>* sr = reinterpret_cast<const SerializedObjectPointers<DataType> *>(data);
-			
-			if (sr) {
-				signal_object_drop (sr->type, sr->cnt, sr->data);
-			}
-			
+		if (selection_data.get_target() == "GTK_TREE_MODEL_ROW") {
+			TreeView::on_drag_data_received (context, x, y, selection_data, info, time);
+		} else if (selection_data.get_target() == object_type) {
+			signal_drop (context, selection_data);
+			context->drag_finish (true, false, time);
 		} else {
 			/* some kind of target type added by the app, which will be handled by a signal handler */
 		}
 	}
 
-  private:
+	/**
+	 * This can be called by the Treeview itself or by some other
+	 * object that wants to get the list of dragged items.
+	 */
 
-	SerializedObjectPointers<DataType>* serialize_pointers (Glib::RefPtr<Gtk::TreeModel> model, 
-								Gtk::TreeSelection::ListHandle_Path* selection,
-								Glib::ustring type) {
+	void get_object_drag_data (std::list<DataType>& l, Gtk::TreeView** source) const {
 
-		/* this nasty chunk of code is here because X's DnD protocol (probably other graphics UI's too) 
-		   requires that we package up the entire data collection for DnD in a single contiguous region
-		   (so that it can be trivially copied between address spaces). We don't know the type of DataType so
-		   we have to mix-and-match C and C++ programming techniques here to get the right result.
-
-		   The C trick is to use the "someType foo[0];" declaration trick to create a zero-sized array at the
-		   end of a SerializedObjectPointers<DataType object. Then we allocate a raw memory buffer that extends
-		   past that array and thus provides space for however many DataType items we actually want to pass
-		   around.
-
-		   The C++ trick is to use the placement operator new() syntax to initialize that extra
-		   memory properly.
-		*/
-		
-		uint32_t cnt = selection->size();
-		uint32_t sz = (sizeof (DataType) * cnt) + sizeof (SerializedObjectPointers<DataType>);
-
-		char* buf = new char[sz];
-		SerializedObjectPointers<DataType>* sr = (SerializedObjectPointers<DataType>*) buf;
-
-		for (uint32_t i = 0; i < cnt; ++i) {
-			new ((void *) &sr->data[i]) DataType ();
-		}
-		
-		sr->cnt = cnt;
-		sr->size = sz;
-		snprintf (sr->type, sizeof (sr->type), "%s", type.c_str());
-		
-		cnt = 0;
-		
-		for (Gtk::TreeSelection::ListHandle_Path::iterator x = selection->begin(); x != selection->end(); ++x, ++cnt) {
-			model->get_iter (*x)->get_value (data_column, sr->data[cnt]);
+		if (drag_data.source == 0) {
+			return;
 		}
 
-		return sr;
+		Glib::RefPtr<Gtk::TreeModel> model = drag_data.source->get_model();
+		DataType v;
+		Gtk::TreeSelection::ListHandle_Path selection = drag_data.source->get_selection()->get_selected_rows ();
+
+		for (Gtk::TreeSelection::ListHandle_Path::iterator x = selection.begin(); x != selection.end(); ++x) {
+			model->get_iter (*x)->get_value (drag_data.data_column, v);
+			l.push_back (v);
+		}
+
+		*source = drag_data.source;
 	}
 };
 
 } // namespace
- 
+
 #endif /* __gtkmm2ext_dndtreeview_h__ */
