@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 1999 Paul Barton-Davis 
+    Copyright (C) 1999 Paul Barton-Davis
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,12 +21,18 @@
 #define __pbd_gtk_ui_h__
 
 #include <string>
-#include <queue>
 #include <map>
 
 #include <stdint.h>
 #include <setjmp.h>
 #include <pthread.h>
+
+#ifdef interface
+#undef interface
+#endif
+
+#include <glibmm/thread.h>
+
 #include <gtkmm/widget.h>
 #include <gtkmm/style.h>
 #ifndef GTK_NEW_TOOLTIP_API
@@ -35,15 +41,17 @@
 #include <gtkmm/textbuffer.h>
 #include <gtkmm/main.h>
 #include <gdkmm/color.h>
-#include <pbd/abstract_ui.h>
-#include <pbd/ringbufferNPT.h>
- 
-#include <pbd/pool.h>
-#include <pbd/error.h>
-#include <pbd/receiver.h>
 
-using std::string;
-using std::queue;
+#ifndef ABSTRACT_UI_EXPORTS
+#define ABSTRACT_UI_EXPORTS
+#endif
+#include "pbd/abstract_ui.h"
+#include "pbd/ringbufferNPT.h"
+#include "pbd/pool.h"
+#include "pbd/error.h"
+#include "pbd/receiver.h"
+
+#include "gtkmm2ext/visibility.h"
 
 class Touchable;
 
@@ -51,8 +59,8 @@ namespace Gtkmm2ext {
 
 class TextViewer;
 
+extern BaseUI::RequestType NullMessage;
 extern BaseUI::RequestType ErrorMessage;
-extern BaseUI::RequestType Quit;
 extern BaseUI::RequestType CallSlot;
 extern BaseUI::RequestType TouchDisplay;
 extern BaseUI::RequestType StateChange;
@@ -60,14 +68,14 @@ extern BaseUI::RequestType SetTip;
 extern BaseUI::RequestType AddIdle;
 extern BaseUI::RequestType AddTimeout;
 
-struct UIRequest : public BaseUI::BaseRequestObject {
-     
+struct LIBGTKMM2EXT_API UIRequest : public BaseUI::BaseRequestObject {
+
      /* this once used anonymous unions to merge elements
 	that are never part of the same request. that makes
 	the creation of a legal copy constructor difficult
 	because of the semantics of the slot member.
      */
-     
+
     Touchable *display;
     const char *msg;
     Gtk::StateType new_state;
@@ -76,20 +84,36 @@ struct UIRequest : public BaseUI::BaseRequestObject {
     Transmitter::Channel chn;
     void *arg;
     const char *msg2;
-    sigc::slot<void> slot;
-    
-    ~UIRequest () { 
+
+    UIRequest () {
+            type = NullMessage;
+    }
+
+    ~UIRequest () {
 	    if (type == ErrorMessage && msg) {
 		    /* msg was strdup()'ed */
-		    free ((char *)msg);
+		    free (const_cast<char *>(msg));
 	    }
     }
- };
+};
 
-class UI : public Receiver, public AbstractUI<UIRequest>
+class LIBGTKMM2EXT_API UI : public AbstractUI<UIRequest>
 {
+  private:
+	class MyReceiver : public Receiver {
+	  public:
+		MyReceiver (UI& ui) : _ui (ui) {}
+		void receive (Transmitter::Channel chn, const char *msg) {
+			_ui.receive (chn, msg);
+		}
+	  private:
+		UI& _ui;
+	};
+
+	MyReceiver _receiver;
+
   public:
-	UI (string name, int *argc, char **argv[]);
+	UI (std::string, std::string, int *argc, char **argv[]);
 	virtual ~UI ();
 
 	static UI *instance() { return theGtkUI; }
@@ -102,57 +126,57 @@ class UI : public Receiver, public AbstractUI<UIRequest>
 
 	bool caller_is_ui_thread ();
 
-	static pthread_t thread_id() { return gui_thread; }
-
 	/* Gtk-UI specific interfaces */
 
 	bool running ();
 	void quit    ();
-	void kill    ();
-	int  load_rcfile (string, bool themechange = false);
+	int  load_rcfile (std::string, bool themechange = false);
 	void run (Receiver &old_receiver);
 
 	void set_state (Gtk::Widget *w, Gtk::StateType state);
-	void popup_error (const char *text);
+	void popup_error (const std::string& text);
 	void flush_pending ();
 	void toggle_errors ();
+	void show_errors ();
+	void dump_errors (std::ostream&);
 	void touch_display (Touchable *);
-	void set_tip (Gtk::Widget *w, const gchar *tip, const gchar *hlp);
+	void set_tip (Gtk::Widget &w, const gchar *tip);
+	void set_tip (Gtk::Widget &w, const std::string &tip);
+	void set_tip (Gtk::Widget *w, const gchar *tip, const gchar *hlp="");
 	void idle_add (int (*func)(void *), void *arg);
 
+	Gtk::Main& main() const { return *theMain; }
+
 	template<class T> static bool idle_delete (T *obj) { delete obj; return false; }
 	template<class T> static void delete_when_idle (T *obj) {
 		Glib::signal_idle().connect (bind (slot (&UI::idle_delete<T>), obj));
 	}
 
-	Gdk::Color get_color (const string& prompt, bool& picked, const Gdk::Color *initial = 0);
+	template<class T> void delete_in_self (T *obj) {
+		call_slot (boost::bind (&UI::delete_in_self, this, obj));
+	}
+
+	Gdk::Color get_color (const std::string& prompt, bool& picked, const Gdk::Color *initial = 0);
 
 	/* starting is sent just before we enter the main loop,
 	   stopping just after we return from it (at the top level)
 	*/
 
-	sigc::signal<void> starting;
-	sigc::signal<void> stopping;
+        virtual int starting() = 0;
 
 	sigc::signal<void> theme_changed;
 
 	static bool just_hide_it (GdkEventAny *, Gtk::Window *);
 
-	static pthread_t the_gui_thread() { return gui_thread; }
-
   protected:
-	bool _auto_display_errors;
 	virtual void handle_fatal (const char *);
-	virtual void display_message (const char *prefix, gint prefix_len, 
-				      Glib::RefPtr<Gtk::TextBuffer::Tag> ptag, 
-				      Glib::RefPtr<Gtk::TextBuffer::Tag> mtag, 
-				      const char *msg);
-	void show_error_log ();
-	void hide_error_log ();
+	virtual void display_message (const char *prefix, gint prefix_len,
+			Glib::RefPtr<Gtk::TextBuffer::Tag> ptag, Glib::RefPtr<Gtk::TextBuffer::Tag> mtag,
+			const char *msg);
 
   private:
 	static UI *theGtkUI;
-	static pthread_t gui_thread;
+
 	bool _active;
 	Gtk::Main *theMain;
 #ifndef GTK_NEW_TOOLTIP_API
@@ -172,11 +196,15 @@ class UI : public Receiver, public AbstractUI<UIRequest>
 	void process_error_message (Transmitter::Channel, const char *);
 	void do_quit ();
 
+	Glib::Threads::Mutex   error_lock;
+	std::list<std::string> error_stack;
+
 	void color_selection_done (bool status);
 	bool color_selection_deleted (GdkEventAny *);
 	bool color_picked;
 
 	void do_request (UIRequest*);
+
 };
 
 } /* namespace */
