@@ -1,6 +1,6 @@
 /*
     Copyright (C) 1998-99 Paul Barton-Davis
- 
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -22,12 +22,14 @@
 
 #include <string>
 
+#include "gtkmm2ext/visibility.h"
+
 namespace Gtk {
 	class Widget;
 }
 
-void gtk_set_size_request_to_display_given_text (Gtk::Widget& w,
-						 const std::string& text,
-						 gint hpadding = 0,
-						 gint vpadding = 0);
+LIBGTKMM2EXT_API void gtk_set_size_request_to_display_given_text (Gtk::Widget& w,
+								  const std::string& text,
+								  gint hpadding = 0,
+								  gint vpadding = 0);
 #endif /* __gtkutils_h__ */
