@@ -1,5 +1,7 @@
-/* 
-   Copyright (C) 2006 Hans Fugal & Paul Davis
+
+/*
+    Copyright (C) 2006 Paul Davis
+    Author: Hans Fugal
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,17 +22,37 @@
 #ifndef __lib_pbd_command_h__
 #define __lib_pbd_command_h__
 
-#include <pbd/statefuldestructible.h>
+#include <string>
 
-class Command : public PBD::StatefulDestructible
+#include "pbd/libpbd_visibility.h"
+#include "pbd/signals.h"
+#include "pbd/statefuldestructible.h"
+
+class LIBPBD_API Command : public PBD::StatefulDestructible, public PBD::ScopedConnectionList
 {
-    public:
-	virtual ~Command() {}
+public:
+	virtual ~Command() { /* NOTE: derived classes must call drop_references() */ }
+
 	virtual void operator() () = 0;
-        virtual void undo() = 0;
-        virtual void redo() { (*this)(); }
-        virtual XMLNode &get_state();
-        virtual int set_state(const XMLNode&) { /* noop */ return 0; }
+
+	void set_name (const std::string& str) { _name = str; }
+	const std::string& name() const { return _name; }
+
+	virtual void undo() = 0;
+	virtual void redo() { (*this)(); }
+
+	virtual XMLNode &get_state();
+	virtual int set_state(const XMLNode&, int /*version*/) { /* noop */ return 0; }
+
+	virtual bool empty () const {
+		return false;
+	}
+
+protected:
+	Command() {}
+	Command(const std::string& name) : _name(name) {}
+
+	std::string _name;
 };
 
 #endif // __lib_pbd_command_h_
